/*
 * %rtf      Copyright Stat-Tech Services, LLC  1998
 *
 *       Please register this product.
 *
 *       For questions or comments please contact:
 *
 *       Stat-Tech Services, LLC
 *       105 N. Crabtree Knoll
 *       Chapel Hill, NC  27514
 *       Fax:  919-493-2776 / Phone:  919-493-2106
 *
 *       Questions - please e-mail stattsrv@aol.com
 *
 *       see http://members.aol.com/stattsrv/stattech.htm or
 *           http://members.aol.com/stattsrv/freemacros.htm for latest versions
 *
 * Purpose-  creates an RTF output file
 * Author-   G. DeMuth
 * Date-     15 Nov 1998
 *
 * Modifications
 *
      16 Nov 1998  Added TAILs to tables.   ged001
                   Added spanning headers.

      jri001    added trim left functions to column to avoid varaibels overflow when format
                not $200 11/22/98 3:36PM

      29 Nov 1998  Added code to deal with left and right text in headers and footnotes. ged002
                   Added top and bottom continued messages handling.
                   Added decimal tab centering as column justification option.

      30 Nov 1998 Fixed problem with continuation of formatting options.  When we altered
                  the title area formatting to use tables - format information was
                  carried through to all cells of the table - causing them to be bold in
                  our example.  Code was added to turn of any formatting that was turned
                  on in the title call, etc.

      7 Dec 1998  Fixed problem with construction of spanning headers.
                  Added special S type of continuation message for bottom of table-
                  appears at right cell of bottom tail message.

      9 Dec 1998  Changed code handling multiple title levels.

      15 Dec 1998  Changed code for SHOWPAGE and addressed problem with
                   use of LINE= variable (from RTFINIT).

      16 Dec 1998 Added two styles:  LINEX and LINEXB, Fixed problem with OUTLINE style.

        jri001 12/16/98 2:44PM added NEWPAGE to mark when past first page.

        jri002 12/21/98 9:16PM added syntax to accept filename from rtfttlgn if _fname
        is missing.

        jri003 12/21/98 9:19PM added syntax to place continuation text on title line
        specified via code in topjust.

      11 Jan 1999   10:07 fixed bottom line condition when at the bottom of the page.

      14 Jan 1999   10:50 altered handling of new page condition to a more robust algorithm.

      18 Jan 1999   9:55p removed {} around all text

      26 Jan 1999   modified LEFT() function added back by JRI to be optional via USELEFT=Y in
                    RTFINIT.

                    ged020  commented out some code that was already addressed in RTFASMBL - also
                            believed this code could cause problems.

      25 Feb 1999  ged implemented &_bypage option (if not Y then no page breaks on BY variable counts)

      7/21/99 8:59AM jri Added box style to line following headers so the headers will no be separated
      from table with blank line.

      8/10/99 11:11AM to prevent last paragraph from creating blank page - jri

      15Jan2000   Updated RTF to support Word 2000 - and be more robust to other Word files.
                  Not robust to WordPerfect v8.

        4/13/00 2:52PM updated to optionally add continuation text to by groups that span two or more
        pages. - jri004

 */

%macro rtf ;

    %local _i _j _ce1 _ce2 _ce3 _ce4 _ce5 _ce6 _ce7
           _ce8 _ce9 _ce10 _ce11 _ce12 _ce13 _ce14
           _ce15 _ce16 _ce17 _ce18 _ce19 _ce20 _celag
           _mlvl _txloc _txlast _txwid ;

    %* If continuation type = S and continuation message is specified, then ;
    %* must have at least one tail footnote - check and issue a blank one ;
    %* if necessary ;
    %if %bquote(&_contmsg) ne and &_ntail = 0 %then %do ;
        %rtftail(fontnum=&_bodyf,fontsize=&_bodys) ;
        %end ;

    %* Close off any unclosed spanning headers - ged001 ;
    %do _i = 1 %to &_clevel ;
        %rtfspend ;
        %end ;

    %* reverse level numbering ;
    %let _mlvl = 0 ;
    %do _i = 1 %to &_nspan ;
        %if &&_slvl&_i > &_mlvl %then %do ;
            %let _mlvl = &&_slvl&_i ;
            %end ;
        %end ;
    %do _i = 1 %to &_nspan ;
        %let _slvl&_i = %eval(&_mlvl - &&_slvl&_i + 1) ;
        %end ;


    %* Calculate column endpoints ;
    %let _ce1 = &_wid1 ;
    %let _celag = &_ce1 ;
    %do _i = 2 %to &_ncol ;
        %let _ce&_i = %eval(&_celag + &&_wid&_i) ;
        %let _celag = &&_ce&_i ;
        %end ;

    %* Determine header heights for columns ;
    data _null_ ;
        length _lab _labtmp $200 ;
        _maxht = 1 ;
        %do _i = 1 %to &_ncol ;
            _lab = trim("&&_lab&_i") ;
            _lablen = length(_lab) ;
            _labtmp = repeat(' ',199) ;
            _tmploc = 0 ;
            _colht = 1 ;
            do _i = 1 to _lablen ;
                _tmploc = _tmploc + 1 ;
                c = substr(_lab,_i,1) ;
                if c = "&_split" then do ;
                    _colht = _colht + 1 ;
                    substr(_labtmp,_tmploc,4) = '\par ' ;
                    _tmploc = _tmploc + 4 ;
                    end ;
                else do ;
                    substr(_labtmp,_tmploc,1) = c ;
                    end ;
                end ;
            call symput("_lab&_i",trim(_labtmp)) ;
            call symput("_colht&_i",compress(put(_colht,2.))) ;
            if _maxht < _colht then _maxht = _colht ;
            %end ;
        call symput('_maxht',compress(put(_maxht,2.))) ;
        run ;

    %* Obtain all sorting variables ;
    %let _sortc = 0 ;
    %do _i = 1 %to &_ncol ;
        %if &&_typ&_i = BY %then %do ;
            %let _sortc = %eval(&_sortc + 1) ;
            %end ;
        %end ;

    %* Sort the data ;
    %if &_page ne or &_sortc > 0 or &_line ne %then %do ;
        proc sort data=&_data ;
            by &_page &_line %do _i = 1 %to &_sortc ;
                                &&_var&_i
                             %end ;
                           ;
            run ;
        %end ;

    %* Map spanning column names to the order of columns ;
    %do _i = 1 %to &_nspan ;
        %do _j = 1 %to &_ncol ;
            %if %upcase(&&_strt&_i) = %upcase(&&_var&_j) %then %do ;
                %let _strt&_i = &_j ;
                %end ;
            %if %upcase(&&_stop&_i) = %upcase(&&_var&_j) %then %do ;
                %let _stop&_i = &_j ;
                %end ;
            %end ;
        %end ;

    %* Fix pagination based on _tabline ;
    %if &_sortc > 0 or &_line ne %then %do ;

        data _cnt ;
            retain _cnt ;
            set &_data ;
            by &_page &_line
               %do _i = 1 %to &_sortc ;
                     &&_var&_i
                     %end ;
                     ;
            %if &_sortc = 0 %then %do ;
                if first.&_line then _cnt = 0 ;
                _cnt = _cnt + 1 ;
                if last.&_line then output ;
                %end ;
            %else %do ;
                if first.&&_var&_sortc then _cnt = 0 ;
                _cnt = _cnt + 1 ;
                if last.&&_var&_sortc then output ;
                %end ;
            keep &_page &_line
                   %do _i = 1 %to &_sortc ;
                        &&_var&_i
                        %end ;
                   _cnt ;
            run ;

        data _next ;
            retain _line 0 ;
            set _cnt ;
            _line = _line + 1 ;
            if _line > 1 ;
            keep _cnt ;
            rename _cnt = _next ;
            run ;

        data _cnt ;
            merge _cnt _next ;
            if _next = . then _next = 0 ;
            run ;

        proc print data=_cnt ;
            run ;

        %end ;

            %global printfil ;


    %* print RTF file ;
    data _null_ ;
        length _value $200 ;
        retain line pline 0 ;
        retain pagtop 1 ;

        /* jri001 12/16/98 2:44PM added to mark when past first page.
        */
        retain newpage 0 ;

        retain pagend 0 ;

        /* jri002 12/21/98 9:16PM added to accept filename from rtfttlgn if _fname is misssing.
        */
        /* ged020  This code already handled correctly in RTFASMBL.MAC
          %if &printfil> %then %do;
                 file "&printfil" noprint notitles  ;
           %end;
           %else %do;
                  file "&_fname" noprint notitles  ;
           %end;
          */
        file "&_fname" noprint notitles  ;

        %if &_line ne or &_sortc > 0 %then %do ;
            merge &_data _cnt end=eof ;
            %end ;
        %else %do ;
            set &_data end=eof ;
            %end ;

        %if &_page ne or &_sortc > 0 or &_line ne %then %do ;
            by &_page &_line %do _i = 1 %to &_sortc ;
                                &&_var&_i
                             %end ;
                           ;
            %end ;

        %* Set up RTF header ;
        if line = 0 then link initrtf ;
        %if &_page ne %then %do ;
            if first.&_page then pagtop = 1 ;
            %end ;
        if pagtop = 1 then link headers ;

        %* Check for need to generate a page break ;
        %if &_line ne and &_sortc = 0 %then %do ;
            if (first.&_line and (pline + _cnt) >= &_tabline and pagtop ne 1) or
               (pagtop ne 1 and pline >= &_tabline) then do ;
            %end ;
        %else %if &_sortc > 0  & &_bypage = Y %then %do ;
            if (first.&&_var&_sortc and (pline + _cnt) >= &_tabline and pagtop ne 1) or
               (pagtop ne 1 and pline >= &_tabline) then do ;
            %end ;
        %else %do ;
            if pline >= &_tabline then do ;
            %end ;
           %if &_ntail > 0 %then %do ;
               link tails ;
               %end ;
           %if &_nfoot > 0 %then %do ;
               link footers ;
               %end ;
           if not eof then do ;
               pagtop = 1 ;
               link headers ;
               end ;
            end ;

        %* Moved from behind following text 1/14/1999 ;
        line = line + 1 ;
        pline = pline + 1 ;

        %* Fix to calculation of pagend - ged 1/14/1999 ;
        %if &_style eq OUTLINE or &_style = LINE or &_style = LINEXB and &_sortc > 0 or &_line > 0 %then %do ;
            if pline >= (&_tabline - 1)
            %if &_sortc > 0  & &_bypage = Y %then %do ;
                or (last.&&_var&_sortc and (pline + _next) >= (&_tabline - 1))
                %end ;
            %else %if &_line ne %then %do ;
                or (last.&_line and (pline + _next) >= (&_tabline - 1))
                %end ;
            then pagend = 1 ;
            %end ;
        %else %do ;
            if pline >= &_tabline
            %if &_sortc > 0  & &_bypage = Y %then %do ;
                or (last.&&_var&_sortc and (pline + _next) > (&_tabline))
                %end ;
            %else %if &_line ne %then %do ;
                or (last.&_line and (pline + _next) > (&_tabline))
                %end ;
            then pagend = 1 ;
            %end ;

        %* GED 12/17/98 ;
        if pagend = 1 or eof or (pline >= &_tabline)
            %if &_page ne %then %do ;
                or last.&_page
                %end ;
            %if &_sortc > 0 or &_line ne and &_style ne BOX &_style ne LINEXB %then %do ;
                or ( %if &_sortc > 0 %then %do ;
                        last.&&_var&_sortc
                        %end ;
                    %else %do ;
                        last.&_line
                        %end ;
                    and (pline + _next) >= &_tabline )
                %end ;
            %else %if &_sortc > 0 or &_line ne %then %do ;  %* 12-15-1998 ;
                or ( %if &_sortc > 0 %then %do ;
                        last.&&_var&_sortc
                        %end ;
                    %else %do ;
                        last.&_line
                        %end ;
                     )
                %end ;
            then do ;
            put "\trowd\trgaph&_linespc\trqc"  ;
            %do _i = 1 %to &_ncol ;
                %if &_style = OUTLINE and &_i = 1 %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    %end ;
                %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = BOX %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                put "\clbrdrb\brdrs\brdrw15\cellx&&_ce&_i" ;
                %end ;
            put "\pard" ;
            %do _i = 1 %to &_ncol ;

                %if &_i <= &_sortc %then %do ;

                        /*jri001*/
                        %if &_uleft = Y %then %do ;
                                if first.&&_var&_i or pagtop = 1 then do;
                                        _value = trim(left(put(&&_var&_i,&&_fmt&_i)))  ;
                                end;
                        %end ;
                        %else %do ;
                                 if first.&&_var&_i or pagtop = 1 then do;
                                       _value = trim(put(&&_var&_i,&&_fmt&_i))  ;
                                end;

                        %end ;
                        else _value = "" ;

                    %end ;
                %else %do ;
                        /*jri001*/
                        %if &_uleft = Y %then %do ;
                                _value = trim ( left( put(&&_var&_i,&&_fmt&_i))) ;

                        %end ;
                        %else %do ;
                                _value = trim ( put(&&_var&_i,&&_fmt&_i)) ;
                        %end ;

                %end ;

                %if &&_just&_i ne d and &&_just&_i ne m %then %do ;
                    %if &&_hang&_i eq 0 %then %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %else %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi-&&_hang&_i\li&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %end ;
                %else %if &&_just&_i eq d %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys\tqdec\tx&_txloc " @@ ;
                    %end ;
                %else %if &&_just&_i eq m %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    %end ;
                _zl = length(_value) ;
                put _value $varying200. _zl  @@;
                put "\cell\pard" ;
                %end ;
            put "\widctlpar\intbl \row \pard" ;

            pagtop = 0 ;  %* Reset pagtop 1/14/1999 ;
            end ;
        else do ;
            put "\trowd\trgaph&_linespc\trqc"  ;
            _lcond = 0 ;
            %do _i = 1 %to &_ncol ;
                %if &_style = OUTLINE and &_i = 1 %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    %end ;
                %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = LINEX %then %do ;
                    %if &_sortc > 0 %then %do ;
                        if last.&&_var&_i or _lcond = 1 then do ;
                            _lcond = 1 ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %else %if &_line ne %then %do ;
                        if last.&_line then do ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %end ;
                %if &_style = BOX %then %do ;
                    %if &_i <= &_sortc %then %do ;
                        if last.&&_var&_i or _lcond = 1 then do ;
                            _lcond = 1 ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %if &_i > &_sortc %then %do ;
                        if _lcond = 1 then do ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %if &_sortc = 0 and &_line ne %then %do ;
                        if last.&_line then do ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                put "\cellx&&_ce&_i" ;
                %end ;
            put "\pard" ;
            %do _i = 1 %to &_ncol ;  /* to deal with type=by variables */
                %if &_i <= &_sortc %then %do ;
                        /*jri001*/
                    %if &_uleft = Y %then %do ;
                        if first.&&_var&_i or pagtop = 1 then do;

                                _value = trim( left( put(&&_var&_i,&&_fmt&_i)));

                                /* jri004 */
                                if pagtop=1 and first.&&_var&_i=0 and "&&_uscnt&_i"="Y" and "&&_typ&_i"="BY" then do;
                                        _value=trim(  left( put(&&_var&_i,&&_fmt&_i)))||' '||trim( left( "&&_cntx&_i"));
                                end;
                        end;

                        %end ;
                    %else %do ;

                        if first.&&_var&_i or pagtop = 1 then do;
                                _value = trim( put(&&_var&_i,&&_fmt&_i));

                                /* jri004 */
                                if pagtop=1 and first.&&_var&_i=0 and "&&_uscnt&_i"="Y" and "&&_typ&_i"="BY" then do;
                                        _value=trim(  put(&&_var&_i,&&_fmt&_i))||' '||trim( left( "&&_cntx&_i"));
                                end;
                        end;

                        %end ;

                    else _value = "" ;
                    %end ;/* end of  dealing with type=by variables */
                %else %do ; /* non-by variables */
                        /*jri001*/
                    %if &_uleft = Y %then %do ;
                        _value = trim( left( put(&&_var&_i,&&_fmt&_i))) ;
                        %end ;
                    %else %do ;
                        _value = trim( put(&&_var&_i,&&_fmt&_i));
                        %end ;
                    %end ;
                %if &&_just&_i ne d and &&_just&_i ne m %then %do ;
                    %if &&_hang&_i eq 0 %then %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %else %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi-&&_hang&_i\li&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %end ;
                %else %if &&_just&_i eq d %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys\tqdec\tx&_txloc " @@ ;
                    %end ;
                %else %if &&_just&_i eq m %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    %end ;
                _zl = length(_value) ;
                put _value $varying200. _zl   ;
                put "\cell\pard" ;
                %end ;
            put "\widctlpar\intbl \row \pard" ;
            pagtop = 0 ;

            end ;/* end of non-by variables */

        %* Blank lines between BY variables ;
        %if (&_sortc > 0 or &_line ne) and &_style ne BOX and &_style ne LINEX %then %do ;
            if
                %if &_line ne and &_sortc = 0 %then %do ;
                    (last.&_line and pagend ne 1 and not eof)
                    %end ;
                %else %if &_line eq and &_sortc > 0 %then %do ;
                    (last.&&_var&_sortc and pagend ne 1 and not eof)
                    %end ;
                %else %if &_line ne and &_sortc > 0 %then %do ;
                    ((last.&_line or last.&&_var&_sortc) and pagend ne 1 and not eof)
                    %end ;
                and (_next + pline) < &_tabline
                   then do ;
                pline = pline + 1 ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                %do _i = 1 %to &_ncol ;
                    %if &_style = OUTLINE and &_i = 1 %then %do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        %end ;
                    %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        %end ;
                    %if &_style = LINEXB %then %do ;
                        put "\clbrdrt\brdrs\brdrw15" @@ ;
                        %end ;
                    %* GED 1/11/1999 fix to bottom border ;
                    if (pline + 1) >= &_tabline or (pline + _next + 1) > &_tabline then do ;
                        put "\clbrdrb\brdrs\brdrw15" @@ ;
                        end ;
                    put "\cellx&&_ce&_i" ;
                    %end ;
                put "\pard" ;
                %do _i = 1 %to &_ncol ;
                    _value = "\cell\pard" ;
                    put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    put _value  ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;

                end ;
            %end ;

        %* Terminates a page ;
        if pagend = 1 or eof
              %if &_page ne %then %do ;
                  or last.&_page
                  %end ;
               then do ;
               pagtop = 1 ;               %* GED 1/14/1999 fix setting of page top ;
               %if &_ntail > 0 %then %do ;
                   link tails ;
                   %end ;
               %if &_nfoot > 0 %then %do ;
                   link footers ;
                   %end ;
            end ;
        /* jri - 8/10/99 11:11AM to prevent last paragraph form creating blank page
        */
        if eof then put '\qc\widctlpar\fi0\f1\fs2 \par }' ;
        %* if eof then put '}' ;
        return ;

        %* RTF file header information ;
        initrtf :
            %* RTF header ;
            put '{\rtf1\ansi' ;
            %* Font table ;
            put '\deff {\fonttbl' ;
            %do _i = 1 %to &_nfonts ;
                put '{' "&&_font&_i" "}" ;
                %end ;
            put '}' ;
            %* Information ;
            put '{\info' ;
            %if %bquote(&_title) ne %then %do ;
                put "{\title &_title }" ;
                %end ;
            %if %bquote(&_auth) ne %then %do ;
                put "{\author &_auth }" ;
                %end ;
            %if %bquote(&_compy) ne %then %do ;
                put "{\company &_compy }" ;
                %end ;
            %if %bquote(&_comm) ne %then %do ;
                put "{\comment &_comm }" ;
                %end ;
            put '}' ;

            %* Margins ;
            %if &_land = Y %then %do ;
                %if &_paper = LEGAL %then %do ;
                    put '\paperw20160\paperh12240\widowctrl\ftnbj \fet0\sectd \lndscpsxn\linex0 '  ;
                    %end ;
                %else %do ;
                    put '\paperw15840\paperh12240\widowctrl\ftnbj \fet0\sectd \lndscpsxn\linex0 '  ;
                    %end ;
                %end ;
            %else %do ;
                %if &_paper = LEGAL %then %do ;
                    put '\paperw12240\paperh20160\widowctrl\ftnbj \fet0\sectd \linex0 '  ;
                    %end ;
                %else %do ;
                    put '\paperw12240\paperh15840\widowctrl\ftnbj \fet0\sectd \linex0 '  ;
                    %end ;
                %end ;
            put "\margl&_leftm" ;
            put "\margr&_rightm" ;
            put "\margt&_topm" ;
            put "\margb&_botm" ;

            return ;

        %* Headers ;
        headers:
            pagtop = 1 ;
            put "\qc\widctlpar\fi0\f1\fs2 \par" ;

            %* ged002 ;
            %* Build table for headers - first assign widths to header area ;
            %* calculate cell margins based on region widths as specified in RTFINIT ;
            %do _i = 1 %to &_nhead ;
                %* determine cell locations for left, center, and right cells ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                put "\cellx&&_hlm&_i" ;
                put "\cellx%eval(&_aleft - &&_hrm&_i)" ;
                put "\cellx&_aleft" ;
                put "\pard" ;

                %* print cell information left, center, and right ;
                %* left text ;
                put "\ql\widctlpar\intbl\fi0\f&&_hfn&_i\fs&&_hfs&_i " @@ ;
                %if &&_hbold&_i = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &&_hunder&_i = Y %then %do ;
                    put "\ul " @@ ;
                    %end;
                %if &&_hital&_i = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                _value = trim("&&_hltxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl  @@ ;
                %if &&_hbold&_i = Y %then %do ;
                    put "\b0" @@ ;
                    %end ;
                %if &&_hunder&_i = Y %then %do ;
                    put "\ul0" @@ ;
                    %end;
                %if &&_hital&_i = Y %then %do ;
                    put "\i0" @@ ;
                    %end ;
                put "\cell\pard" ;

                %* center text ;
                put "\qc\widctlpar\intbl\fi0\f&&_hfn&_i\fs&&_hfs&_i " @@ ;
                %if &&_hbold&_i = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &&_hunder&_i = Y %then %do ;
                    put "\ul " @@ ;
                    %end;
                %if &&_hital&_i = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                _value = trim("&&_htxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl  @@ ;

                        /* jri003 12/21/98 9:19PM added to place continuation text on title line
                           specified via code in topjust.
                        */
                        %if &_topjst=t %then %do;
                                %if %eval(&_i=&_topline) %then %do;
                                        if newpage=1 then do;
                                                put "&_topmsg" @@;
                                        end;
                                %end;
                        %end;

                %if &&_hbold&_i = Y %then %do ;
                    put "\b0" @@ ;
                    %end ;
                %if &&_hunder&_i = Y %then %do ;
                    put "\ul0" @@ ;
                    %end;
                %if &&_hital&_i = Y %then %do ;
                    put "\i0" @@ ;
                    %end ;
                put "\cell\pard" ;

                %* right text ;
                put "\qr\widctlpar\intbl\fi0\f&&_hfn&_i\fs&&_hfs&_i " @@ ;
                %if &&_hbold&_i = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &&_hunder&_i = Y %then %do ;
                    put "\ul " @@ ;
                    %end;
                %if &&_hital&_i = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                _value = trim("&&_hrtxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl  @@ ;
                %if &&_hbold&_i = Y %then %do ;
                    put "\b0" @@ ;
                    %end ;
                %if &&_hunder&_i = Y %then %do ;
                    put "\ul0" @@ ;
                    %end;
                %if &&_hital&_i = Y %then %do ;
                    put "\i0" @@ ;
                    %end ;
                put "\cell\pard" ;
                put "\widctlpar\intbl \row \pard" ;
            %end ;

            %* Page by label ;
            %if &_page ne and &_pages ne N %then %do ;
                put "\qc\widctlpar\fi0\f&&_hfn&_nhead\fs&&_hfs&_nhead \par " @ ;
                %if &&_hbold&_nhead = Y %then %do ;
                    put '\b ' @@ ;
                    %end ;
                %if &&_hunder&_nhead = Y %then %do ;
                    put '\ul ' @@ ;
                    %end ;
                %if &&_hital&_nhead = Y %then %do ;
                    put '\i ' @@ ;
                    %end ;
                _value = trim("&_pagel") || ' ' || trim(put(&_page,&_pagef)) ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                put '\par\par' @@ ;
                %if &&_hbold&_nhead = Y %then %do ;
                    put '\b0 ' @@ ;
                    %end ;
                %if &&_hunder&_nhead = Y %then %do ;
                    put '\ul0 ' @@ ;
                    %end ;
                %if &&_hital&_nhead = Y %then %do ;
                    put '\i0 ' @@ ;
                    %end ;
                %end ;

            %* Top of table continued message ;
            %if %bquote(&_topmsg) ne and &_topjst^=t%then %do ;
                %if &_topfp ne Y %then %do ;
                    if not (line = 0) then do ;
                    %end ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                put "\cellx&&_ce&_ncol" ;
                put "\pard" ;
                put "\q&_topjst\widctlpar\intbl\fi0\f&_labf\fs&_labs " @ ;
                _value = trim("&_topmsg") ;
                _zl = length(_value) ;
                put _value $varying200. _zl ;
                put '\cell\pard'  ;
                put "\widctlpar\intbl \row \pard" ;

                %if &_topfp ne Y %then %do ;
                    end ;
                    %end ;
                %end ;

            %* Spanning headers ;
            %* Get maximum depth of spanning headers ;
            %let _depth = 0 ;
            %do _i = 1 %to &_nspan ;
                %if &&_slvl&_i > &_depth %then %do ;
                    %let _depth = &&_slvl&_i ;
                    %end ;
                %end ;

            %do _i = 1 %to &_depth ;
                %let _c = %eval(&_depth - &_i + 1) ;
                %* Spanning headers are assumed ordered - pick them out to create the line ;
                %* For each spanning header put blank cells as necessary ;
                %let _tots = 0 ;
                %do _j = 1 %to &_nspan ;
                    %if &&_slvl&_j = &_c %then %do;
                        %let _tots = %eval(&_tots + 1) ;
                        %end ;
                    %end ;
                %* %put _TOTS &_tots ;

                %* First make cell definitions ;
                %let _currcol = 0 ;
                %let _lagcnt = 0 ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                %do _j = 1 %to &_nspan ;
                    %* Find if spanning column matches current depth ;
                    %if &&_slvl&_j = &_c %then %do ;
                        %let _lagcnt = %eval(&_lagcnt + 1) ;
                        %let _currend = &&_stop&_j ;
                        %* Line information ;
                        %if &_style = OUTLINE and &_currcol = 0 %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            %end ;
                        %* %if &_style = OUTLINE and &_lagcnt = &_tots %then %do ;
                            %* put "\clbrdrr\brdrs\brdrw15" @@ ;
                            %* %end ;
                        %if &_style = BOX and &_currcol = 0 %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            *put "\clbrdrb\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_i = 1 %then %do ;
                            put "\clbrdrt\brdrs\brdrw15" @@ ;
                            %end ;
                        %* Find if need adjust for column location ;
                        %if &_currcol ne %eval(&&_strt&_j - 1) %then %do ;
                            %* Fix currcol by making blank cell number equal to stop column ;
                            %let _currcol = %eval(&&_strt&_j - 1) ;
                            put "\cellx&&_ce&_currcol" ;
                            %end ;
                        %let _currcol = &&_stop&_j ;
                        %put _currcol = &_currcol   _lagend = &_lagcnt   ;
                        %* Make cell with borders as needed ;
                        %if &_style = OUTLINE and &_currcol = 0 %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_style = OUTLINE and &_currend = &_ncol %then %do ;  %* change to cell 12/16/98 ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_style = BOX and &_currcol = 0 %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            *put "\clbrdrb\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_style = BOX and &_currend = &_ncol %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            *put "\clbrdrb\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_i = 1 %then %do ;
                            put "\clbrdrt\brdrs\brdrw15" @@ ;
                            %end ;
                        %let _tmpc = &&_stop&_j ;
                        %if &_style = NOBOTTOMBORDER %then %do;
                                put "\cellx&&_ce&_tmpc" ;
                        %end;
                        %else %do;
                                put "\clbrdrb\brdrs\brdrw15\cellx&&_ce&_tmpc" ;
                        %end;
                        %end ;
                    %end ;
                %* fix for currend condition - add extra cell ;
                %if &_currend ne &_ncol %then %do ;
                    %if &_style = OUTLINE %then %do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        %end ;
                    %if &_style = BOX %then %do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        *put "\clbrdrb\brdrs\brdrw15" @@ ;
                        %end ;
                    %if &_i = 1 %then %do ;
                        put "\clbrdrt\brdrs\brdrw15" @@ ;
                        %end ;
                    %let _tmpc = &&_stop&_j ;
                    put "\cellx&&_ce&_ncol" ;
                    %end ;
                put "\pard" ;

                %* Write entries for each cell ;
                %let _currcol = 0 ;
                %do _j = 1 %to &_nspan ;
                    %* Find if spanning column matches current depth ;
                    %if &&_slvl&_j = &_c %then %do ;
                        %let _currend = &&_stop&_j ;
                        %* Find if need adjust for column location ;
                        %if &_currcol ne %eval(&&_strt&_j - 1) %then %do ;
                            %* Fix currcol by making blank cell number equal to stop column ;
                            %let _currcol = %eval(&&_strt&_j - 1) ;
                            put "\qc\widctlpar\intbl\fi0\f&_labf\fs&_labs \cell\pard" ;
                            %end ;
                        %* Make cell with borders as needed ;
                        put "\qc\widctlpar\intbl\fi0\f&_labf\fs&_labs " @@ ;
                        %if &_labb = Y %then %do ;
                            put "\b " @@ ;
                            %end ;
                        %if &_labu = Y %then %do ;
                            put "\ul " @@ ;
                            %end ;
                        %if &_labi = Y %then %do ;
                            put "\i " @@ ;
                            %end ;
                        _value = trim("&&_stxt&_j") ;
                        _zl = length(_value) ;
                        put _value $varying200. _zl @@ ;
                        %if &_labb = Y %then %do ;
                            put "\b0 " @@ ;
                            %end ;
                        %if &_labu = Y %then %do ;
                            put "\ul0 " @@ ;
                            %end ;
                        %if &_labi = Y %then %do ;
                            put "\i0 " @@ ;
                            %end ;
                        put "\cell\pard" ;
                        %let _currcol = &&_stop&_j ;
                        %end ;
                    %end ;
                %if &_currend ne &_ncol %then %do ;
                    put "\qc\widctlpar\intbl\fi0\f&_labf\fs&_labs \cell\pard" ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;

                %end ;


            %* Table headers ;
            put "\trowd\trgaph&_linespc\trqc"  ;
            %do _i = 1 %to &_ncol ;
                %if &_nspan < 1 %then %do ;
                    put "\clbrdrt\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = OUTLINE and &_i = 1 %then %do ;
                    %let _lagcol = &_currcol ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    %end ;
                %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = BOX %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                        %if &_style = NOBOTTOMBORDER %then %do;
                                put "\cellx&&_ce&_i" ;
                        %end;
                        %else %do;
                                put "\clbrdrb\brdrs\brdrw15\cellx&&_ce&_i" ;
                        %end;
                %end ;
            put "\pard" ;
            %do _i = 1 %to &_ncol ;
                %if &&_just&_i = d or &&_just&_i = m %then %do ;
                    put "\qc\widctlpar\intbl\fi&&_hang&_i\f&_labf\fs&_labs " @@ ;
                    %end ;
                %else %do ;
                    put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_labf\fs&_labs " @@ ;
                    %end ;
                %if &_labb = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &_labu = Y %then %do ;
                    put "\ul " @@ ;
                    %end ;
                %if &_labi = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                %let _ch = &&_colht&_i ;
                %if &_ch < &_maxht %then %do ;
                    %let _c = %eval(&_maxht - &_ch) ;
                    %do _j = 1 %to &_c ;
                        put '\par ' @@ ;
                        %end ;
                    %end ;
                _value = trim("&&_lab&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                %if &_labb = Y %then %do ;
                    put "\b0 " @@ ;
                    %end ;
                %if &_labu = Y %then %do ;
                    put "\ul0 " @@ ;
                    %end ;
                %if &_labi = Y %then %do ;
                    put "\i0 " @@ ;
                    %end ;
                put "\cell\pard" ;
                %end ;
            put "\widctlpar\intbl \row \pard" ;

        %if &_hedskip=Y %then %do;

                pline = pline + 1 ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                %do _i = 1 %to &_ncol ;
                    %if &_style = OUTLINE and &_i = 1 %then %do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        %end ;
                    %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        %end ;
                %if &_style = BOX %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                    put "\cellx&&_ce&_i" ;
                    %end ;
                put "\pard" ;
                %do _i = 1 %to &_ncol ;
                    _value = "\cell\pard" ;
                    put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    put _value  ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;

        %end;

                /* jri001 12/16/98 2:44PM added to mark when past first page.
                */
                if newpage=0 then newpage=1;

            return ;

        %* Tails ;
        tails:
            %if %bquote(&_contmsg) ne and &_contjst ne s %then %do ;
                %if &_contlp ne Y %then %do ;
                    if not eof then do ;
                    %end ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                put "\cellx&&_ce&_ncol" ;
                put "\pard" ;
                put "\q&_contjst\widctlpar\intbl\fi0\f&_bodyf\fs&_bodys " @ ;
                _value = trim("&_contmsg") ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                put "\cell\pard" ;
                put "\widctlpar\intbl \row \pard" ;

                %if &_contlp ne Y %then %do ;
                    end ;
                    %end ;
                %end ;

            %do _i = 1 %to &_ntail ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                %if %bquote(&_contmsg) ne and &_contjst eq s and &_i = &_ntail %then %do ;
                    %let _tmp = %eval(&&_ce&_ncol - 144*15) ;
                    put "\cellx&_tmp" ;
                    put "\cellx&&_ce&_ncol" ;
                    %end ;
                %else %do ;
                    put "\cellx&&_ce&_ncol" ;
                    %end ;
                put "\pard" ;
                put "\q&&_tjst&_i\widctlpar\intbl\fi0\f&&_tfn&_i\fs&&_tfs&_i " @ ;
                %if &&_tbold&_i = Y %then %do ;
                    put '\b ' @@ ;
                    %end ;
                %if &&_tunder&_i = Y %then %do ;
                    put '\ul ' @@ ;
                    %end ;
                %if &&_tital&_i = Y %then %do ;
                    put '\i ' @@ ;
                    %end ;
                _value = trim("&&_ttxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                %if &&_tbold&_i = Y %then %do ;
                    put '\b0 ' @@ ;
                    %end ;
                %if &&_tunder&_i = Y %then %do ;
                    put '\ul0 ' @@ ;
                    %end ;
                %if &&_tital&_i = Y %then %do ;
                    put '\i0 ' @@ ;
                    %end ;
                put "\cell\pard" ;
                %if %bquote(&_contmsg) ne and &_contjst = s and &_i = &_ntail %then %do ;
                    put "\qr\widctlpar\intbl\fi0\f&&_tfn&_i\fs&&_tfs&_i " @ ;
                    %if &&_tbold&_i = Y %then %do ;
                        put '\b ' @@ ;
                        %end ;
                    %if &&_tunder&_i = Y %then %do ;
                        put '\ul ' @@ ;
                        %end ;
                    %if &&_tital&_i = Y %then %do ;
                        put '\i ' @@ ;
                        %end ;
                    %if &_contlp ne Y %then %do ;
                        if not eof then do ;
                        %end ;
                       _value = trim("&_contmsg") ;
                    %if &_contlp ne Y %then %do ;
                            end ;
                        else do ;
                            _value = '' ;
                            end ;
                        %end ;
                    _zl = length(_value) ;
                    put _value $varying200. _zl @@ ;
                    %if &&_tbold&_i = Y %then %do ;
                        put '\b0 ' @@ ;
                        %end ;
                    %if &&_tunder&_i = Y %then %do ;
                        put '\ul0 ' @@ ;
                        %end ;
                    %if &&_tital&_i = Y %then %do ;
                        put '\i0 ' @@ ;
                        %end ;
                    put "\cell\pard" ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;

               %end;

            %if &_nfoot = 0 %then %do ;
                pagend = 0 ;
                pline = 0 ;
                if not eof then do ;
                    put "{\f1 \column }" ;
                    end ;
                %end ;
            return ;

        %* Footers ;
        footers:
            pagend = 0 ;
            do _i = pline to &_tabline ;
                put '\par' ;
                end ;
            pline = 0 ;
            %* ged002 ;
            %* Build table for headers - first assign widths to header area ;
            %* calculate cell margins based on region widths as specified in RTFINIT ;
            %do _i = 1 %to &_nfoot ;
                %* determine cell locations for left, center, and right cells ;
                put "\trowd\trgaph&_linespc\trqc"  ;
                put "\cellx&&_flm&_i" ;
                put "\cellx%eval(&_aleft - &&_frm&_i)" ;
                put "\cellx&_aleft" ;
                put "\pard" ;

                %* print cell information left, center, and right ;
                %* left text ;
                put "\ql\widctlpar\intbl\fi0\f&&_ffn&_i\fs&&_ffs&_i " @@ ;
                %if &&_fbold&_i = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &&_funder&_i = Y %then %do ;
                    put "\ul " @@ ;
                    %end;
                %if &&_fital&_i = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                _value = trim("&&_fltxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                %if &&_fbold&_i = Y %then %do ;
                    put "\b0" @@ ;
                    %end ;
                %if &&_funder&_i = Y %then %do ;
                    put "\ul0" @@ ;
                    %end;
                %if &&_fital&_i = Y %then %do ;
                    put "\i0" @@ ;
                    %end ;
                put "\cell\pard" ;

                %* center text ;
                put "\q&&_fjst&_i\widctlpar\intbl\fi0\f&&_ffn&_i\fs&&_ffs&_i " @@ ;
                %if &&_fbold&_i = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &&_funder&_i = Y %then %do ;
                    put "\ul " @@ ;
                    %end;
                %if &&_fital&_i = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                _value = trim("&&_ftxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl  @@ ;
                %if &&_fbold&_i = Y %then %do ;
                    put "\b0" @@ ;
                    %end ;
                %if &&_funder&_i = Y %then %do ;
                    put "\ul0" @@ ;
                    %end;
                %if &&_fital&_i = Y %then %do ;
                    put "\i0" @@ ;
                    %end ;
                put "\cell\pard" ;

                %* right text ;
                put "\qr\widctlpar\intbl\fi0\f&&_ffn&_i\fs&&_ffs&_i " @@ ;
                %if &&_fbold&_i = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &&_funder&_i = Y %then %do ;
                    put "\ul " @@ ;
                    %end;
                %if &&_fital&_i = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                _value = trim("&&_frtxt&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                %if &&_fbold&_i = Y %then %do ;
                    put "\b0" @@ ;
                    %end ;
                %if &&_funder&_i = Y %then %do ;
                    put "\ul0" @@ ;
                    %end;
                %if &&_fital&_i = Y %then %do ;
                    put "\i0" @@ ;
                    %end ;
                put "\cell\pard" ;
                put "\widctlpar\intbl \row \pard" ;
		%end ;

            if not eof then do ;
                put "{\f1 \column }" ;
                end ;
            return ;

        run ;

%mend rtf ;


