/* rtfasmbl.mac Copyright 1998 Stat-Tech Services, LLC 

    Macro Family:  Table Generation System

    Purpose:    Compiles the summary data sets and prints the table as an RTF file

    Required Macros:  RTF system macros:

                        %rtf %rtfcol %rtffoot %rtfinit %rtftail %rtftitle %rtfspan %rtfspend

    Parameters:
    
                Parameters are a combination of those associated with assemble in the MRS
                and those associated with the rtf system, most notably the parameters
                associated with rtfinit.
                

        Name        Definition
        --------------------------


        george: see questions marked jriq.                              

*/

%macro rtfasmbl(

            /* assemble macro control parameters */
            
                format=COMBINED,     /* or LABELSEP format of table */
                labelind=3,          /* GED003 Default label indent */
                tspace=Y,            /* or Y, blank line generator  */
                floc=BOTTOM,         /* Footnote location/top or bottom */
                skip=P,              /* Defaults of paginating in summaries */
                dumpgrp=Y,
                contxt=,
                conlp=N,             
                banner=Y,
                headline=Y,
                overline=Y,
                divline=N,
                addset1=,
                addlab1=,
                addset2=,
                addlab2=,
                addset3=,
                addlab3=,
                addset4=,
                addlab4=,
                addset5=,
                addlab5=,
            
            /* rtf macro control parameters */
            
                style=lines,                     /* Lines, Outline, Boxes */
                
                contmsg=,    /* Continued message (bottom of table) */
                contlp=N,                        /* Continued message on last page */
                contjst=s,
                
                topmsg=,     /* Continued message (top of table) */
                topfp=N,                         /* Continued message on first page */
                topjst=L,                       /* Justification of top continued message (R,L,C t#) */
                tablines=40,                     /* Lines per page within Body */
                
                headskip=y,                     /* skip line in table after header jri 1/20/99 8:29AM*/

                bodyfont=3,                      /* Body font # and font size */
                bodysize=10,
                
                labfont=3,                       /* Label font # and font size */
                labsize=10,
                labbold=n,
                labital=N,                       /* Label special font control */
                labunder=N,
                land=Y,
                line=_morder,                    /* Line variable */
                
                topm=15,                         /* margins top, left, right, bottom (tenths of inches) */
                bottom=10,
                leftm=10,
                rightm=10,
                
                filname=,
                
                DRAFT=n
            
            ) ;

    %local ;
    
    
        /* counter variable for special spanning headers.
        */
        %global
                _specspn
                ;

    %* get file name ;
    %if %scan(&filname,1) ne %then %do ;
        %let pagfile = &filname ;
        %end ;
    %else %do ;
        %let pagfile = &printfil ;
        %end ;


    %let style = %upcase(&style) ;

    %* ;
    %* Count additional data sets ;
    %* ;
    %let _addc = 0 ;
    %if %scan(&addset1,1) ne %then %let _addc = %eval(&_addc + 1) ;
    %if %scan(&addset2,1) ne %then %let _addc = %eval(&_addc + 1) ;
    %if %scan(&addset3,1) ne %then %let _addc = %eval(&_addc + 1) ;
    %if %scan(&addset4,1) ne %then %let _addc = %eval(&_addc + 1) ;
    %if %scan(&addset5,1) ne %then %let _addc = %eval(&_addc + 1) ;

    %* ;
    %* This code is used to get &_pg and &_by values attached to note data sets ;
    %* ;
    %if &_pg ne or &_by ne %then %do ;
        data _tmpgb ;
            set %do _i = 1 %to &_varcnt ;
                    _iset&_i 
                    %end ; ;
            if _isnote ne 1 ;
            keep &_by &_pg ;
            run ;

        proc sort data=_tmpgb ;
            by &_pg &_by ;
            run ;

        data _tmpgb;
            set _tmpgb ;
            by &_pg &_by ;
            %if &_by ne %then %do ;
                if first.&_by ;
                %end ;
            %else %do ;
                if first.&_pg ;
                %end ;

        data _null_ ;
            set _tmpgb end=eof ;
            if eof then call symput('_totobs',compress(_n_)) ;
            run ;
    
        %end ;

    %* ;
    %* Collapse all summary data sets and separate out total column ;
    %* ;
    %* Combine data sets and look for _AECLASS information ;
    %* Also calculate first column number if _AECLASS = 1 ;
    %let _aeclass = 0 ;
    %let _firstc = 0 ;  %* GED016 ;
    data _combdat ;
    
                
        set %do _i = 1 %to &_varcnt ;
                _iset&_i 
                %end ; 
            %if %scan(&addset1,1) ne %then %do ;
                &addset1 &addset2 &addset3 &addset4 &addset5
                %end ;
            end = eof ;
        retain _firstc . ;
        if _colnum ne . and _firstc = . then _firstc = _colnum ;
        else if _colnum ne . then do ;
            if _firstc > _colnum then _firstc = _colnum ;
            end ;
        if eof then call symput('_firstc',compress(_firstc)) ;
        
                /* tags cells as numeric or character to implement 
                   decimal and center justification in rtf.mac
                   
                   JRI 12/2/98 6:27PM
                */
                length
                        _tabflg         3.
                        _tmpvar         8.
                        ;
                _tmpvar=input( _col, ?? 8.);
                
                if _tmpvar=. then _tabflag=1; /* character (center) */
                else _tabflag=0; /* numeric (decimal) */
                
	    run ;	


    %if &_firstc = %then %do ;   %* GED016 ;
        %let _firstc = 0 ;
        %end ;

    data _tmpcol _tmptot _add1 _add2 _add3 _add4 _add5 ;
    

        retain _colcnt 0 ;

                length
                        _col    $200.
                        _jcol   $200.
                        _chknum 8.
                        ;

        set _combdat end = eof ;
        if _col^='' then do;
                _chknum=input( trim( left(_col)), ?? 8.);

                if _chknum=. then do;
                        if index( _col, '(' )>0 then do;

            _jcol='\tqdec\tx'||trim(left(int(144*&_cwtwips*.40)))||' \tqr\tx'||trim(left(int(144*&_cwtwips*.80)))||
                  ' {\tab '||trim( left( substr( _col, 1, index( _col, '(' )-1)));

            _col=trim( left( _jcol))||' \tab '||compress(substr( _col, index( _col, '(' ) ), ' ')||'}';
                        
                        end;
                        else do;
                                _col='\tqc\tx'||trim(left(int(144*&_cwtwips*.50)))||' {'||trim( left(_col))||'}';
                        end;
                end;
                if _chknum>. then do;
                        _col = '\tqdec\tx'||trim(left(int(144*&_cwtwips*.50)))||' {'||trim( left(_col))||'}';
                end;
        end;

        %if %upcase(%substr(&draft,1,1)) = Y %then %do ;   %* Draft = yes function ;
              _col = ' ' ;
              %end ;

        %* GED012 additions for _AECLASS ;
        if _aeclass = 1 then call symput('_aeclass','1') ;
        if _colnum = . then _colnum = &_firstc ;
        if _colcnt < _colnum then _colcnt = _colnum ;
        if eof then call symput('_aecols',compress(_colcnt)) ;
        
        if _total = 0 then output _tmpcol ;
        else if _total = 1 then output _tmptot ;
        if _addcol = 1 then output _add1 ;
        else if _addcol = 2 then output _add2 ;
        else if _addcol = 3 then output _add3 ;
        else if _addcol = 4 then output _add4 ;
        else if _addcol = 5 then output _add5 ;

        run ;


    %* DUMPGRP option removes extra columns when GRP and SUBGRP analyses are ;
    %* present, but the user does not want the extra columsn to appear ;
    %if &_grp ne and %substr(%upcase(&dumpgrp),1,1) = Y %then %do ;

        proc sort data=_tmpcol ;
            by &_grp &_sgrp ;
            run ;

        data _dump ;
            retain _keep . ;
            set _tmpcol ;
            by &_grp &_sgrp ;
            %if &_sgrp ne %then %do ;
                if first.&_sgrp then do ;
                %end ;
            %else %do ;
                if first.&_grp then do ;
                %end ;
                _keep = 0 ;
                end ;
            if not (trim(left(_col)) in (' ' '0')) then _keep = 1 ;
            %if &_sgrp ne %then %do ;
                if last.&_sgrp then do ;
                %end ;
            %else %do ;
                if last.&_grp then do ;
                %end ;
                if _keep = 1 then output ;
                end ;
            keep &_grp &_sgrp ;
            run ;

        data _tmpcol ;
            merge _tmpcol
                  _dump (in=indump) ;
            by &_grp &_sgrp ;
            if indump ;
            run ;

        proc datasets library=work ;
            delete _dump ;
            quit ;

        %end ;
        
    %* ;
    %* This code adds extra records to the total data for notes when a _pg or _by ;
    %* variable is present ;
    %* ;
    %if &_pg ne or &_by ne %then %do ;
        data _tmptot ;
            set _tmptot ;
            if _isnote = 1 then do ;
                do _i = 1 to &_totobs ;
                    set _tmpgb point=_i ;
                    output;
                    end ;
                end ;
            else do ;
                output ;
                end ;
            run ;
        %end ;

    %* If _AECLASS is present on the data set - extract the column headers ;
    %* And store them as macro variables - GED012 ;
    %if &_aeclass = 1 %then %do ;
        proc sort data=_tmptot out=_tmphdr ;
            by _colnum ;
            run ;

        data _null_ ;
            set _tmphdr end=eof ;
            by _colnum ;
            if _collab ne ' ' then call symput('_aeh' || compress(_colnum),_collab) ;
            run ;

        %do _i = 1 %to &_aecols ;
            %put HEADER &_i = &&_aeh&_i ;
            %end ;
            
        %end ;
        
    
    %* ;
    %* Look at presence of _grp or _sgrp to generate columns - get total ;
    %* column count ;
    %* ;
    %let _colcnt = 0 ;
    %if &_grp ne or &_sgrp ne %then %do ;

        %* obtain total merge groups ;
        proc freq data=_tmpcol noprint ;
            table &_grp %if &_sgrp ne %then %do ;
                            *&_sgrp
                            %end ;
                  / out=_mset noprint ;
            run ;

        %* get total column count and keep an output counter ;
        %* Generate column headers based on _grp/_sgrp levels ;
        data _mset ;
            set _mset (keep=&_grp &_sgrp) end=eof ;
            _coln = _n_ ;
            %* headers for _grp only ;
            %if &_grp ne and &_sgrp eq %then %do ;
                call symput('_colh' || compress(_n_),put(&_grp,&_grpf)) ;
                %end ;
            %else %do ;
                call symput('_sph' || compress(_n_),put(&_grp,&_grpf)) ;
                call symput('_colh' || compress(_n_),put(&_sgrp,&_sgrpf)) ;
                %end ;
            if eof then call symput('_colcnt',compress(_n_)) ;
            run ;

        %* merge _mset against _tmpcol to obtain columns ;
        proc sort data=_tmpcol ;
            by &_grp &_sgrp ;
            run ;
        
        data %do _i=1 %to &_colcnt ;
                _col&_i
                %end ; ;
            merge _tmpcol _mset ;
            by &_grp &_sgrp ;
            %do _i = 1 %to &_colcnt ;
                if _coln = &_i then output _col&_i ;
                %end ;
            run ;

        %end ;

    %if &_aeclass = 1 %then %do ;
        %let _mxae = %eval(&_colcnt * &_aecols) ;
        %end ;
    %else %do ;
        %let _mxae = 0 ;
        %end ;

    %* Merge all output files together ;
    %if &_colcnt = 0 %then %do ;
    
        proc sort data=_tmptot ;
            by &_pg &_by _morder _varcnt _rorder ;
            run ;
            
        data _pset ;
            set _tmptot (rename=(_col=_totcol)) ;
            by &_pg &_by _morder _varcnt _rorder ;
            %* GED012 - if _AECLASS is 1 transpose columns ;
            %if &_aeclass = 1 %then %do ;
                length _colt1-_colt&_aecols $60. ;
                retain tcols {*} _colt1-_colt&_aecols ;
                if first._rorder then do ;
                    do _i = 1 to &_aecols ;
                        tcols{_i} = ' ' ;
                        end ;
                    end ;
                tcols{_colnum} = _col ;
                if last._rorder then output ;
                drop _i ;
                %end ;
            run ;
            
        %end ;
    %else %do ;

        %* ged018 additional code to include _COLNUM variable ;
        proc sort data=_tmptot ;
            by &_pg &_by _morder _varcnt _rorder
                %if &_aeclass %then %do ;
                    _colnum
                    %end ; ;
            run ;

        %do _i = 1 %to &_colcnt ;
            proc sort data=_col&_i ;
                by &_pg &_by _morder _varcnt _rorder
                    %if &_aeclass %then %do ;
                        _colnum
                        %end ;
                     ;
                run ;
            %end ;
            
        data _pset ;
            merge _tmptot (rename=(_col=_totcol)) 
                  %do _i = 1 %to &_colcnt ;
                        _col&_i (keep=&_pg &_by _morder _varcnt _rorder _col
                                    %if &_aeclass %then %do ;
                                        _colnum
                                        %end ;        
                                      rename=(_col=_col&_i))
                        %end ; ;
            by &_pg &_by _morder _varcnt _rorder
                %if &_aeclass %then %do ;
                    _colnum
                    %end ;
                ;
            %* GED012 - if _AECLASS is 1 transpose columns ;
            %if &_aeclass = 1 %then %do ;
                length _colt1-_colt&_aecols $60 ;
                length _ct_1-_ct_&_mxae $60 ;
                retain _colt1-_colt&_aecols ;
                retain _ct_1-_ct_&_mxae ;
                array tcols {*} _colt1-_colt&_aecols ;
                %do _i = 1 %to &_colcnt ;
                    %let _s = %eval(&_aecols*%eval(&_i - 1) + 1) ;
                    %let _e = %eval(&_s + &_aecols - 1) ;
                    array a&_i.cols {*} _ct_&_s - _ct_&_e ;
                    %end ;
                if first._rorder then do ;
                    do _i = 1 to &_aecols ;
                        tcols{_i} = ' ' ;
                        %do _i = 1 %to &_colcnt ;
                            a&_i.cols{_i} = ' ' ;
                            %end ;
                        end ;
                    end ;
                tcols{_colnum} = _totcol ;
                %do _i = 1 %to &_colcnt ;
                    a&_i.cols{_colnum} = _col&_i ;
                    %end ;
                if last._rorder then output ;
                drop _i ;
                %end ;
            run ;
            %end ;

    %* Add additional columns ;
    %do _i = 1 %to &_addc ;
        %if &&addset&_i ne %then %do ;
            proc sort data=_add&_i ;
                by &_pg &_by _morder _varcnt _rorder ;
                run ;

            data _pset ;
                merge _pset _add&_i (keep=&_pg &_by _morder _varcnt _rorder _col rename=(_col=_add&_i)) ;
                by &_pg &_by _morder _varcnt _rorder ;
                run ;
            %end ;
        %end ;

    %* Clean-up labels for printing (e.g. do wrapping) ;
    %if %upcase(&format) = COMBINED %then %do ;
    
        data _pset ;
            set _pset ;
            by &_pg &_by _morder _varcnt _rorder ;
            length _char _char1-_char8 $100. ; * $&_vwid.. ;
            array _chars{*} _char1-_char8 ;
            length _totc $60. ;
            %if &_colcnt > 0 %then %do ;
                %if &_aeclass = 0 %then %do ;
                    length _tcol1-_tcol&_colcnt $60. ;
                    array _cols {*} _col1-_col&_colcnt ;
                    array _tcols {*} _tcol1-_tcol&_colcnt ;
                    %end ;
                %else %do ;
                    length _tcol1-_tcol&_mxae $60. ;
                    array _cols {*} _ct_1-_ct_&_mxae ;
                    array _tcols {*} _tcol1-_tcol&_mxae ;
                    length _tcolt1-_tcolt&_aecols $60. ;
                    array _ttmps {*} _tcolt1-_tcolt&_aecols ;
                    array _tacts {*} _colt1-_colt&_aecols ;
                    %end ;
                %end ;
            %if &_addc > 0 %then %do ;
                length _acol1-_acol&_addc $60. ;
                array _adds {*} _add1-_add&_colcnt ;
                array _acols {*} _acol1-_acol&_colcnt ;
                %end ;
            %* parse the variable label ;
            _l = length(_varlab) ;
            if _varlab eq '' then do ;               %* Ged fix when no-label present ;
                _ml = 0 ; 
                _char1 = ' ' ;
          	end ;
            else if _l <= &_vwid then do ;
                _char1 = _varlab ;
                _ml = 1 ;
                end ; 
            else do ;
                _fr = 0 ;   %* GED013 ;
                _sl = 1 ;
                _cp = 0 ;
                _ml = 0 ;
                do while (_cp < _l) ;
                    _ml = _ml + 1 ;
                    _cp = _cp + &_vwid - _fr*&labelind - 1 ;    %* GED007 / GED013 ;
                    if _cp > _l then _cp = _l ;
                    if _cp < _l then do ;
                        do while (substr(_varlab,_cp,1) ne ' ' and _cp > _sl) ;
                            _cp = _cp - 1 ;
                            end ;
                        end ;
                    if _cp = _sl then _cp = _cp + &_vwid - _fr*&labelind - 1 ; %* GED013 ;
                    if _cp > _l then _cp = _l ;
                    if _fr = 0 then do ;
                        _char1 = substr(_varlab,_sl,_cp-_sl+1) ;
                        end ;
                    else do ;
                        %* _chars{_ml} = repeat(' ',3) || substr(_varlab,_sl,_cp-_sl+1) ; %* GED007 ;
                        substr(_chars{_ml},&labelind+1,_cp-_sl+1) = substr(_varlab,_sl,_cp-_sl+1) ;
                        end ;
                    if _fr = 0 then _fr = 1 ;
                    _sl = _cp + 1 ;
                    _cp = _sl ;
                    end ;
                end ;

            if first._varcnt then do ;
                %if &_aeclass = 0 %then %do ;
                    _totc = _totcol ;
                    _totcol = ' ' ;
                    %end ;
                %else %do ;
                    do _i = 1 to &_aecols ;
                        _ttmps{_i} = _tacts{_i} ;
                        _tacts{_i} = ' ' ;
                        end ;
                    %end ;
                %if &_colcnt > 0 %then %do ;
                    %if &_aeclass = 0 %then %do ;
                        do _i = 1 to &_colcnt ;
                            _tcols{_i} = _cols{_i} ;
                            _cols{_i} = ' ' ;
                            end ;
                        %end ;
                    %else %do ;
                        do _i = 1 to &_mxae ;
                            _tcols{_i} = _cols{_i} ;
                            _cols{_i} = ' ' ;
                            end ;
                        %end ;
                    %end ;
                %if &_addc > 0 %then %do ;
                    do _i = 1 to &_addc ;
                        _acols{_i} = _adds{_i} ;
                        _adds{_i} = ' ' ;
                        end ;
                    %end ;
                do _i = 1 to _ml ;
                    _char = _chars{_i} ;
                    if _i < _ml or _rowlab ne ' ' then output ;
                    else if _rowlab = ' ' then _rowlab = _chars{_ml} ;
                    end ;
                %if &_aeclass = 0 %then %do ;
                    _totcol = _totc ;
                    %end ;
                %else %do ;
                    do _i = 1 to &_aecols ;
                        _tacts{_i} = _ttmps{_i} ;
                        end ;
                    %end ;
                %if &_colcnt > 0 %then %do ;
                    %if &_aeclass = 0 %then %do ;
                        do _i = 1 to &_colcnt ;
                            _cols{_i} = _tcols{_i} ;
                            end ;
                        %end ;
                    %else %do ;
                        do _i = 1 to &_mxae ;
                            _cols{_i} = _tcols{_i} ;
                            end ;
                        %end ;
                    %end ;
                %if &_addc > 0 %then %do ;
                    do _i = 1 to &_addc ;
                        _adds{_i} = _acols{_i} ;
                        end ;
                    %end ;
                end ;
                
                /* Do indentation for values and summaries.
                */
                %if &aesum=0 %then %do;
                        if length( trim( _rowlab))=length( trim( left( _rowlab))) then do;
                                _char = trim(left(_rowlab ));
                        end;
                        else do;
                                _char = '\tl\tx360 \tab \tab ' ||trim(left(_rowlab ));
                        end;
                %end;
                %if &aesum=1 %then %do;
                        _char=_rowlab;
                %end;
                                
            if _rowlab ne ' '
                %if &_aeclass = 0 %then %do ;
                    or _totcol ne ' '
                    %do _i = 1 %to &_colcnt ;
                        or _col&_i ne ' '
                        %end ;
                    %end ;
                %else %do ;
                    %do _i = 1 %to &_aecols ;
                        or _colt&_i ne ' '
                        %end ;
                    %do _i = 1 %to &_mxae ;
                        or _col&_i ne ' '
                        %end ;
                    %end ;
                %do _i = 1 %to &_addc ;
                    or _add&_i ne ' '
                    %end ;
                then do ;
                    output ;
                    end ;

            drop _l _cp _sl _fr _i _ml _totc
                 _char1-_char8
                 %if &_colcnt > 0 %then %do ;
                    %if &_aeclass = 0 %then %do ;
                        _tcol1-_tcol&_colcnt
                        %end ;
                    %else %do ;
                        _tcolt1-_tcolt&_aecols
                        _tcol1-_tcol&_mxae
                        %end ;
                    %if &_addc > 0 %then %do ;
                        _acol1-_acol&_colcnt
                        %end ;
                    %end ;
                 ;

            run;

        %end ;
    %else %do ;
        data _pset ;
            set _pset ;
            by &_pg &_by _morder _varcnt _rorder ;
	        if not first._varcnt then _varlab = ' ' ;
	        run ;
    	%end ;

    proc print data=_pset ;
        run ;

    %* correct value column for _pset when using LABELSEP ;
    %if &format = LABELSEP %then %do ;  %* GED002 ;

        data _null_ ;
            set _pset end=eof ;

            retain _minind 200 ;
            do _i = 1 to length(_rowlab) ;
                if substr(_rowlab,_i,1) ne ' ' then do ;
                    _minind = min(_minind,_i) ;
                    end ;
                end ;

            if eof then call symput('_minind',compress(put(_minind,3.))) ;
            
            run ;

        data _pset ;
            set _pset ;

            _rowlab = substr(_rowlab,&_minind,length(_rowlab) - &_minind + 2) ;

            run ;

        %end ;

    %* write the print macro ;
    %if &format = COMBINED %then %do ;

        %if &_colcnt = 0 %then %let _fcol = 0 ;
        %else %let _fcol = 1 ;

        %do %while (&_fcol <= &_colcnt) ;

            %let _lcol = %eval(&_fcol + &_mxcol - 1) ;
            %if &_lcol > &_colcnt %then %do ;
                %let _lcol = &_colcnt ;
                %end ;

                /* rtf initialization call jri 11/17/98 9:48PM */
                %rtfinit( data=_pset, page=&_pg, pagelbl=%bquote(&_pglbl), pagefmt=&_pgfmt, 
                        filename=&pagfile, split=&_split, tablines=&tablines, 
                        bodyfont=&bodyfont, bodysize=&bodysize, labfont=&labfont, 
                        labsize=&labsize,  labbold=&labbold, labital=&labital, 
                        labunder=&labunder, land=&land, line=&line, topm=&topm, 
                        bottom=&bottom, leftm=&leftm, rightm=&rightm,
                        style=&style, contmsg=%bquote(&contmsg), contlp=&contlp, contjst=&contjst, 
                        topmsg=%bquote(&topmsg), topfp=&topfp, topjst=&topjst,
                          
                       headskip=&headskip,
                        
                       titlinit=&_tgenflg /* flag to curtail initialization of _nhead and _nfoot by rtfinit - jri 12/3/98 7:22PM*/
                        
                        );
                        
                        
            %if &_by ne %then %do ;


                /* JRI001
                */
                %rtfcol(var=&_by, fmt=&_byfmt, label=%bquote(&_bylbl), just=L, width=&_bytwips, type=BY, hang=0 ) ; 

                %end ;

                /* JRI001
                */
            
           %rtfcol(var=_char, fmt=$200., label=%bquote(&_vlbl), just=l, width=&_vwtwips, type=, hang=3 ) ; 

               

            %if &_grp ne and &_sgrp eq %then %do ;

                %* Group with no sub-group ;
                %if %bquote(&_grpl) ne %then %do ;
                
                        /* jriq001 we should probably make the font controlable from the 
                                rtfasmbl call.  
                        */
                    %rtfspan(text=%bquote(&_grpl), 
                        fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;
                    %end ;

                %if &_aeclass = 0 %then %do ;
                    %do _i = &_fcol %to &_lcol ;
                    
                    
                        /* implement special spanning header macros calls.
                        */
                        %if %bquote(&_grpl)= %then %do ; 
                                %if &_specspn> %then %do _sp=1 %to &_specspn;
                                        %if %eval(&&_spst&_sp)=&_i %then %do;
                                                %rtfspan(text=%bquote(&&_sptx&_sp), fontnum=&&_spfn&_sp, fontsize=&&_spfs&_sp, 
                                                        just=&&_spjs&_sp, bold=&&_spbd&_sp, under=&&_spun&_sp, italics=&&_spit&_sp) ;
                                        %end;
                                %end;
                        %end;
                        *%rtfcol(var=_col&_i, fmt=$&_cwid.., label=%bquote(&&_colh&_i), just=d, width=&_cwtwips, type=, hang=0) ;
                        %rtfcol(var=_col&_i, fmt=$100., label=%bquote(&&_colh&_i), just=m, width=&_cwtwips, type=, hang=0) ;
                          %end ;
                          
                        %if %bquote(&_grpl)= %then %do ; 
                                %if &_specspn> %then %do _sp=1 %to &_specspn;
                                        %if %eval(&&_spsp&_sp)=&_i %then %do;
                                                %rtfspend;
                                        %end;
                                %end;
                        %end;
                                
                    %rtfspend;
                    
                    %end ;

                    
                %else %do ;
                    %do _i = &_fcol %to &_lcol ;
                    
                        %rtfspan(text=%bquote(&&_colh&_i), 
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;
                                
                        %let _s = %eval(1 + %eval(&_aecols * %eval(&_i - 1))) ;
                        %do _j = &_firstc %to &_aecols ;
                            %let _k = %eval(&_s + &_j - 1) ;
                            /* JRI001
                            */

                           * %rtfcol(var=_ct_&_k, fmt=$&_cwid.., label=%bquote(&&_aeh&_j), just=d, width=&_cwtwips, type=, hang=0) ;
                            %rtfcol(var=_ct_&_k, fmt=$100., label=%bquote(&&_aeh&_j), just=m, width=&_cwtwips, type=, hang=0) ;

                            %end ;

                        %rtfspend;

                        %end ;
                    %end ;
                    
                %end ;
            %else %if &_grp ne %then %do ;

                %* Group with a sub-group ;

                %let _lcf = &&_sph&_fcol ;

                %do _i = &_fcol %to &_lcol ;

                    %if &_i = &_fcol %then %do ;
                    
                        %rtfspan(text=%bquote(&&_sph&_i), 
                               fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;
                                
                        %end ;
                    %else %if %quote(&_lcf) ne %quote(&&_sph&_i) %then %do ;
                        %rtfspend;
                        %rtfspan(text=%bquote(&&_sph&_i), 
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;
                    
                         %let _lcf = &&_sph&_i ;
                        %end ;

                    %if &_aeclass = 0 %then %do ;
                            /* JRI001
                            */

                            *%rtfcol(var=_col&_i, fmt=$&_cwid.., label=%bquote(&&_colh&_i), just=d, width=&_cwtwips, type=, hang=0) ;
                            %rtfcol(var=_col&_i, fmt=$100., label=%bquote(&&_colh&_i), just=m, width=&_cwtwips, type=, hang=0) ;

                        %end ;
                    %else %do ;

                        %rtfspan(text=%bquote(&&_colh&_i)), 
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;


                        %let _s = %eval(1 + &_aecols*%eval(&_i - 1)) ;
                        %do _j = &_firstc %to &_aecols ;
                            %let _k = %eval(&_s + &_j - 1) ;
                            /* JRI001
                            */

                            *%rtfcol(var=_ct_&_k, fmt=$&_cwid.., label=%bquote(&&_aeh&_j), just=d, width=&_cwtwips, type=, hang=0) ;
                            %rtfcol(var=_ct_&_k, fmt=$100., label=%bquote(&&_aeh&_j), just=m, width=&_cwtwips, type=, hang=0) ;

                            %end ;
                        %rtfspend;
                        %end ;

                     %if &_i = &_lcol %then %do ;
                        %end ;

                     %end ;

                %rtfspend;
                %end ;

            %if &_aeclass = 0 %then %do ;
                %if %upcase(%substr(&_tot,1,1)) = Y %then %do ;
                            /* JRI001
                            */

                   *%rtfcol(var=_totcol, fmt=$&_cwid.., label=%bquote(&_totlb), just=d, width=&_cwtwips, type=, hang=0) ;
                    %rtfcol(var=_totcol, fmt=$100., label=%bquote(&_totlb), just=m, width=&_cwtwips, type=, hang=0) ;

                    %end ;
                %end ;
            %else %if &_fcol = &_colcnt %then %do ;
                %if %upcase(%substr(&_tot,1,1)) = Y %then %do ;
                    
                        %rtfspan(text=%bquote(&&_totlb)), 
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;
                    
                    %do _j = 1 %to &_aecols ;
                            /* JRI001
                            */

                        *%rtfcol(var=_colt&_j, fmt=$&_cwid.., label=%bquote(&&_aeh&_j), just=d, width=&_cwtwips, type=, hang=0) ;
                        %rtfcol(var=_colt&_j, fmt=$100., label=%bquote(&&_aeh&_j), just=m, width=&_cwtwips, type=, hang=0) ;

                        %end ;
                    %end ;
                %rtfspend;
                %end ;

            %do _i = 1 %to &_addc ;
            /* jri001
            */

                *%rtfcol(var=_add&_i, fmt=$&_cwid.., label=%bquote(&&addlab&_i), just=d, width=&_cwtwips, type=, hang=0) ;
                %rtfcol(var=_add&_i, fmt=$100., label=%bquote(&&addlab&_i), just=m, width=&_cwtwips, type=, hang=0) ;

                %end ;
                

                %rtf;
                
            %let _fcol = %eval(&_lcol + 1) ;
            
            %end ;

        %end ;
        
    %else %do ;

        %* Column separated ;
        %if &_colcnt = 0 %then %let _fcol = 0 ;
        %else %let _fcol = 1 ;

        %do %while (&_fcol <= &_colcnt) ;

            %let _lcol = %eval(&_fcol + &_mxcol - 1) ;
            %if &_lcol > &_colcnt %then %do ;
                %let _lcol = &_colcnt ;
                %end ;

        /*JRI001
        */
            %if &_by ne %then %do ;

                %rtfcol(var=&_by, fmt=&_byfmt, label=%bquote(&_bylbl), just=l, width=&_cwtwips, type=BY, hang=0) ;

               %end ;
               

                %rtfcol(var=_varlab, fmt=$200., label=%bquote(&_vlbl), just=l, width=&_vwtwips, type=, hang=0) ;


                %rtfcol(var=_rowlab, fmt=$200., label=%bquote(&_vallbl), just=l, width=&_valwid, type=, hang=0) ;


            %if &_grp ne and &_sgrp eq %then %do ;

                %if %bquote(&_grpl) ne %then %do ;

                    %rtfspan(text=%bquote(&_grpl), level=1, 
                        fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;

                    %end ;

                %if &_aeclass = 0 %then %do ;
                    %do _i = &_fcol %to &_lcol ;
        /*JRI001
        */

                *%rtfcol(var=_col&_i, fmt=$&_cwid.., label=%bquote(&&_colh&_i), just=c, width=&_cwtwips, type=, hang=0) ;
                %rtfcol(var=_col&_i, fmt=$100., label=%bquote(&&_colh&_i), just=c, width=&_cwtwips, type=, hang=0) ;

                        %end ;
                    %end ;
                %else %do ;
                    %do _i = &_fcol %to &_lcol ;

                    %rtfspan(text=%bquote(&_colh&_i), 
                        fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;

                        %let _s = %eval(1 + &_aecols*%eval(&_i - 1)) ;
                        %do _j = &_firstc %to &_aecols ;
                            %let _k = %eval(&_s + &_j - 1) ;
        /*JRI001
        */

                            *%rtfcol(var=_ct_&_k, fmt=$&_cwid.., label=%bquote(&&_aeh&_j), just=d, width=&_cwtwips, type=, hang=0) ;
                            %rtfcol(var=_ct_&_k, fmt=$100., label=%bquote(&&_aeh&_j), just=m, width=&_cwtwips, type=, hang=0) ;

                           %end ;
                        %end ;
                    %end ;
                %rtfspend;
                %end ;
            %else %if &_grp ne %then %do ;

                %let _lcf = &&_sph&_fcol ;

                %do _i = &_fcol %to &_lcol ;

                    %if &_i = &_fcol %then %do ;

                            %rtfspan(text=%bquote(&_sph&_i), 
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;

                        %end ;
                    %else %if %quote(&_lcf) ne %quote(&&_sph&_i) %then %do ;

                            %rtfspan(text=%bquote(&_sph&_i)
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;

                        %let _lcf = &&_sph&_i ;
                        %end ;

                    %if &_aeclass = 0 %then %do ;
                    /*JRI001
                    */

                        *%rtfcol(var=_col&_i, fmt=$&_cwid.., label=%bquote(&&_colh&_i), just=d, width=&_cwtwips, type=, hang=0) ;
                        %rtfcol(var=_col&_i, fmt=$100., label=%bquote(&&_colh&_i), just=m, width=&_cwtwips, type=, hang=0) ;

                        %end ;
                    %else %do ;

                            %rtfspan(text=%bquote(&_colh&_i)
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;

                        %let _s = %eval(1 + &_aecols*%eval(&_i - 1)) ;
                        %do _j = &_firstc %to &_aecols ;
                            %let _k = %eval(&_s + &_j - 1) ;
                    /*JRI001
                    */

                           *%rtfcol(var=_ct_&_k, fmt=$&_cwid.., label=%bquote(&&_aeh&_j), just=d, width=&_cwtwips, type=, hang=0) ;
                           %rtfcol(var=_ct_&_k, fmt=$100., label=%bquote(&&_aeh&_j), just=m, width=&_cwtwips, type=, hang=0) ;

                            %end ;
                        %end ;

                     %end ;

                %rtfspend;
                %end ;

            %if &_aeclass = 0 %then %do ;
                %if %upcase(%substr(&_tot,1,1)) = Y %then %do ;
                    /*JRI001
                    */

                   *%rtfcol(var=_totcol, fmt=$&_cwid.., label=%bquote(&_totlb), just=d, width=&_cwtwips, type=, hang=0) ;
                    %rtfcol(var=_totcol, fmt=$100., label=%bquote(&_totlb), just=m, width=&_cwtwips, type=, hang=0) ;

                    %end ;
                %end ;
            %else %if &_fcol = &_colcnt %then %do ;
                %if %upcase(%substr(&_tot,1,1)) = Y %then %do ;

                            %rtfspan(text=%bquote(&_totlb),
                                fontnum=1, fontsize=10, just=c, bold=N, under=N, italics=N) ;

                    %do _j = 1 %to &_aecols ;

                        *%rtfcol(var=_colt&_j, fmt=$&_cwid.., label=%bquote(&&_aeh&_j), just=d, width=&_cwtwips, type=, hang=0) ;
                        %rtfcol(var=_colt&_j, fmt=$100., label=%bquote(&&_aeh&_j), just=m, width=&_cwtwips, type=, hang=0) ;

                        %end ;
                    %end ;
                %rtfspend;
                %end ;

            %do _i = 1 %to &_addc ;

                *%rtfcol(var=_add&_i, fmt=$&_cwid.., label=%bquote(&&addlab&_i), just=d, width=&_cwtwips, type=, hang=0) ;
                %rtfcol(var=_add&_i, fmt=$100., label=%bquote(&&addlab&_i), just=m, width=&_cwtwips, type=, hang=0) ;

                %end ;

                %rtf;
                
            %let _fcol = %eval(&_lcol + 1) ;
            
            %end ;

        %end ;

        /* Initialize special spanner to zero.
        */
        %let _specspn=0;


    %* Clean-up working data sets ;
    proc datasets library=work ;
        delete _tmptot _tmpcol _mset _combdat
               %if &_aeclass = 1 %then %do ;
                    _tmphdr
                    %end ;
               %if &_colcnt > 0 %then %do ;
                    %do _i = 1 %to &_colcnt ;
                        _col&_i
                        %end ;
                    %end ;
               ;
        run ;

    %put LEAVING MACRO ASSEMBLE ;


%mend rtfasmbl ;
