/*
 * %rtfinit   Copyright Stat-Tech Services, LLC
 *
 *       Please register this product.
 *
 *       For questions or comments please contact:
 *
 *       Stat-Tech Services, LLC
 *       105 N. Crabtree Knoll
 *       Chapel Hill, NC  27514
 *       Fax:  919-493-2776 / Phone:  919-493-2106
 *
 *       Questions - please e-mail stattsrv@aol.com
 *
 *       see http://www.stattechservices.com/rtf.htm
 *
 * Purpose-  initializes and RFT output file
 * Author-   G. DeMuth
 * Date-     15 Nov 1998
 *
 * Modifications
 *
 *    20 Nov 1998  - Added global parameter _clevel for syntax change in RTFSPAN
 *    29 Nov 1998 - ged - added continued text and global parameter holding
 *                  the total page size for writing.

      12/3/98 7:18PM jri001  added titlinit=0 if zero title and footnote counters will initialize, if one then
        initialization is expected to have previously occurred. - jri 12/3/98 7:17PM12/3/98 7:17PM

      jri 12/17/98 9:24AM added continuation message option t# where t indicates title and # is title line number.
                          Line number defaults to 1.

      1/26/99  added useleft=Y function to suppress or allow left() on column values.

      2/25/99  added BYPAGE=Y option to suppress pagination on BY conditions if set to N]

      4/13/00 2:56PM fixed parameter mistatement involving useleft parameter.  The global macro variable assignment
      was not correct because useleft had no ampersand. - jri

 */

%macro rtfinit(data=,      /* data set to print */
               filename=tmp.rtf, /* Output file name */
               tablines=10,      /* Lines per page within Body */
               headskip=y,      /* skip line after header */
               bodyfont=3,       /* Body font # and font size */
               bodysize=10,
               labfont=3,        /* Label font # and font size */
               labsize=10,
               labbold=Y,
               labital=N,        /* Label special font control */
               labunder=N,
               land=Y,
               useleft=Y,
               style=lines,      /* Lines, Outline, Boxes */
               page=,            /* Pagination variable */
               pagelbl=,
               pagefmt=,
               bypage=Y,         /* Setting to N turns off pagination on BY information */
               contmsg=%bquote((Continued)),         /* Continued message (bottom of table) */
               contlp=N,         /* Continued message on last page */
               contjst=R,        /* Justification of continued message (R,L,C,S) */
               topmsg=,          /* Continued message (top of table) */
               topfp=N,          /* Continued message on first page */
               topjst=L,         /* Justification of top continued message (R,L,C,T# (title)) */
               showpage=Y,       /* Suppress printing of pagination variable */
               line=,            /* Line variable */
               split=*,
               title=,           /* information fields */
               author=,
               company=,
               comment=,
               paper=STAND,   /* or LEGAL */
               topm=10,      /* margins top, left, right, bottom (tenths of inches) */
               bottom=10,
               leftm=10,
               rightm=10,
               font1=SAS Monospace,     /* Font table information */
               family1=fmodern,
               font2=New Times Roman,
               family2=froman,
               font3=Arial,
               family3=fswiss,
               font4=Symbol,
               family4=ftech,
               font5=,
               family5=,
               titlinit=0       /* jri001 if zero title and footnote counters will initialize, if one then
                                 initialization is expected to have previously occurred.
                                 - jri 12/3/98 7:17PM12/3/98 7:17PM
                                 */
               ) ;

    %local _i ;

    %global _data _title _auth _compy _comm _topm _botm _leftm _rightm
            _font1 _font2 _font3 _font4 _font5 _nfonts
            _nhead _nfoot _ntail _ncol _linespc _fname _tabline
            _bodyf _bodys _labf _labs _labb _labi _labu _land
            _split _page _pagel _pagef _pages _nspan _line _style
            _clevel _aleft _contmsg _contlp _contjst
            _topmsg _topfp _topjst _topline _uleft
            _paper  _bypage
            _hedskip

            ;

    %* Set values of global values to null to clear old information ;
    %let _data = &data ;
    %let _fname = &filename ;
    %let _tabline = &tablines ;
    %let _bodyf = &bodyfont ;
    %let _bodys = %eval(2*&bodysize) ;
    %let _labf = &labfont ;
    %let _labs = %eval(2*&labsize) ;
    %let _labb = %substr(%upcase(&labbold),1,1) ;
    %let _labu = %substr(%upcase(&labunder),1,1) ;
    %let _labi = %substr(%upcase(&labital),1,1) ;
    %let _land = %upcase(%substr(&land,1,1)) ;
    %let _split = &split ;
    %let _page = &page ;
    %let _pagel = &pagelbl ;
    %let _pagef = &pagefmt ;
    %let _pages = %substr(%upcase(&showpage),1,1) ;
    %let _bypage = %substr(%upcase(&bypage),1,1) ;
    %let _line = &line ;
    %let _uleft = %upcase(%substr(&useleft,1,1)) ;
    %let _title = &title ;
    %let _auth = &author ;
    %let _compy = &company ;
    %let _comm = &comment ;
    %let _topm = ;
    %let _botm = ;
    %let _leftm = ;
    %let _rightm = ;
    %let _font1 = ;
    %let _font2 = ;
    %let _font3 = ;
    %let _font4 = ;
    %let _font5 = ;
    %let _nfonts = ;
    %let _aleft = ;
    %let _paper = &paper ;
    %let _linespc = 100 ;

    %let _hedskip=%substr(%upcase(&headskip),1,1) ;

    /* jri001 special exception used by rtf title subsystem - jri 12/3/98 7:19PM
    */
    %if &titlinit=0 %then %do;
            %let _nhead = 0 ;
            %let _nfoot = 0 ;
            %let _ntail = 0 ;
    %end;

    %let _ncol = 0 ;
    %let _nspan = 0 ;
    %let _clevel = 0 ;
    %let _style = %upcase(&style) ;
    %let _contmsg = &contmsg ;
    %let _contlp = %upcase(%substr(&contlp,1,1)) ;
    %let _contjst = %lowcase(%substr(&contjst,1,1)) ;
    %let _topmsg = &topmsg ;
    %let _topfp = %upcase(%substr(&topfp,1,1)) ;
    %let _topjst = %lowcase(%substr(&topjst,1,1)) ;

        /* jri 12/17/98 9:57AM parse line number if topline=t
        */
        %if &_topjst=t %then %do;
                %let _topline=%substr( &topjst, 2);
                %if &_topline= %then %let _topline=1;
        %end;
    data _null_ ;
        %* Margins ;
        size = int(144*&topm) ;
        if size = 0 then size = 720 ;
        call symput('_topm',compress(put(size,5.))) ;
        size = int(144*&bottom) ;
        if size = 0 then size = 720 ;
        call symput('_botm',compress(put(size,5.))) ;
        size = int(144*&leftm) ;
        if size = 0 then size = 720 ;
        call symput('_leftm',compress(put(size,5.))) ;
        size = int(144*&rightm) ;
        if size = 0 then size = 720 ;
        call symput('_rightm',compress(put(size,5.))) ;
        %* Create variables with relative area for headers (port vs land) ;
        %if &_land = Y %then %do ;
            _papwid = 15840 ;
            %end ;
        %else %do ;
            _papwid = 12240 ;
            %end;
        _aleft = _papwid - max(720,144*&leftm) - max(720,144*&rightm) ;
        call symput('_aleft',compress(put(_aleft,5.))) ;
        %* Create font table statments ;
        length fonttxt $80. ;
        nfonts = 0 ;
        %do _i = 1 %to 5 ;
            %if %bquote(&&font&_i) ne and &&family&_i ne %then %do ;
                nfonts = nfonts + 1 ;
                fonttxt = '\f' || "&_i" || '\f' || "&&family&_i" ||
                          '\charset0\fprg2 ' || "&&font&_i" || ';' ;
                call symput("_font&_i",trim(fonttxt)) ;
                %end ;
            %end ;
        call symput('_nfonts',compress(put(nfonts,1.))) ;
        run ;

%mend rtfinit ;


