/*
    Macro RTFPAGE         Paginates an RTF file

    Copyrighted by Stat-Tech Services, LLC   1998

    Macro Family:  Title Management System

    Purpose:    Paginate a file

    Required Macros:  None

    Parameters:         FILE=                    output file reference
    
                        TARGET=Page ### of ###   specifies the target in the
                                                 file used for counting
                                                 pages.
                                                 
                        STYLE=PAGEOF, PAREN      Sytle for pagination:

                                                 PAGEOF = Page XXX or YYY

                                                 PAREN = (XXX/YYY)

                        PP=N                     PP=ON, Y, or YES generates
                                                 a PROC PRINTTO command


    Author:     G DeMuth
    Date:       27 Jan 1999

    Note:  If FILE= is missing it uses the value of &printfil

    Adapted from PAGEIT.MAC


*/

%macro rtfpage (file=,target=PAGE ### OF ###,style=PAGEOF) ;

    %* get file name ;
    %if %scan(&file,1) ne %then %do ;
        %let pagfile = &file ;
        %end ;
    %else %do ;
        %let pagfile = &printfil ;
        %end ;

    %* get file into data set and count appearances of target string ;
    data _pagein _totpag (keep=pagecnt) ;            %* count lines with target ;
            retain pagecnt 0 ;
            length inline $200. ;
            infile "&pagfile" length=ll end=fin pad ;
            input @1 inline $varying200. ll ;
            if index(inline,"&target") > 0 then pagecnt = pagecnt + 1 ;
            output _pagein ;
            if fin then output _totpag ;
            run ;

    data _null_ ;                                %* overwrite existing file ;
            retain maxpage 0 ;
            length pagestr $30. ;
            file "&pagfile" noprint notitles  ;
            set _totpag (in=in1) _pagein ;
            if in1 then maxpage = pagecnt ;
            else do ;
                if index(inline,"&target") > 0 then do ;
            
                    *** write first part of string ;
                    indloc = index(inline,"&target") ;
                    targlen = length("&target") ;
                    length templine $200. ;
                    templine = trim(substr(inline,1,indloc-1)) ;
                    tlen = length(templine) ;
                    put templine $varying200. tlen @@ ;
                    
                    *** put in Page of information ;
                    if upcase("&style") = "PAREN" then do ;
                        pagestr = '(' || compress(put(pagecnt,10.))
                                  || '/' || compress(put(maxpage,10.)) || ')' ;
                        end ;
                    else do ;
                        pagestr = 'Page ' || compress(put(pagecnt,10.))
                                  || ' of ' || compress(put(maxpage,10.)) ;
                        end; 
                    pagestr = trim(pagestr) ;
                    pagel = length(trim(pagestr)) ;
                    put ' ' @@ ;
                    put pagestr $varying40. pagel @@ ;

                    templine = trim(substr(inline,indloc+targlen,length(inline)-indloc-targlen+1)) ;
                    tlen = length(templine) ;
                    put templine $varying200. tlen ;
                    
                    end ;
                else do ;
                    tlen = length(trim(inline)) ;
                    put inline $varying200. tlen ;
                    end ;
                end ;
                
            run ;

%mend rtfpage ;
