/*
 * %rtfprofi   Copyright Stat-Tech Services, LLC, 1999
 *
 *       For questions or comments please contact:
 *
 *       Stat-Tech Services, LLC
 *       105 N. Crabtree Knoll
 *       Chapel Hill, NC  27514
 *       Fax:  919-493-2776 / Phone:  919-493-2106
 *
 *       Questions - please e-mail stattsrv@aol.com
 *
 *       see http://www.stattechservices.com/rtf.htm
 * 
 * Purpose-  initializes an RFT profile file
 * Author-   G. DeMuth
 * Date-     28 Feb 1999
 * 
 * Modifications
 *
 */

%macro rtfprofi(filename=tmp.rtf, /* Output file name */
                land=Y,
                paper=STAND,   /* or LEGAL */
                title=,           /* information fields */
                author=,
                company=,
                comment=,
                topm=1.5,   /* margins top, left, right, bottom (tenths of inches) */
                bottom=1,
                leftm=1,
                rightm=1,
                font1=SAS Monospace,     /* Font table information */
                family1=fmodern,   
                font2=New Times Roman,
                family2=froman,
                font3=Arial,
                family3=fswiss,
                font4=Symbol,
                family4=ftech,
                font5=,
                family5=,
                ) ;

    %local _i ;

    %global _fname _linespc ;
    %let _linespc = 100 ;

    %* Set values of global values to null to clear old information ;
    %let _fname = &filename ;

    %* This data null writes RTF header information ;
    %* print RTF file ;
    data _null_ ;
        length _value $200 ;
        file "&_fname" noprint notitles  ;
        
        %* RTF header ;
        put '{\rtf1\ansi' ;
        
        %* Font table ;
        put '\deff {\fonttbl' ;
            %do _i = 1 %to 5 ;
                %if %bquote(&&font&_i) ne and %bquote(&&family&_i) ne %then %do ;
                    _value = trim('\f' || "&_i" || '\f' || "&&family&_i" ||
                              '\charset0\fprg2 ' || "&&font&_i" || ';') ;
                    put _value @@ ;
                    %end ;
                %end ;
        put '}' ;
       
        %* Information ;
        put '{\info' ;
        %if %bquote(&title) ne %then %do ;
            put "{\title &title }" ;
            %end ; 
        %if %bquote(&author) ne %then %do ;
            put "{\author &author }" ;
            %end ; 
        %if %bquote(&company) ne %then %do ;
            put "{\company &company }" ;
            %end ; 
        %if %bquote(&comment) ne %then %do ;
            put "{\comment &comment }" ;
            %end ;
        put '}' ;

        %* Margins ;
        %if %upcase(%substr(&land,1,1)) = Y %then %do ;
            %if %upcase(%substr(&paper,1,1)) = LEGAL %then %do ;
                put '\paperw20160\paperh12240\widowctrl\ftnbj \fet0\sectd \lndscpsxn\linex0 '  ;
                %end ;
            %else %do ;
                put '\paperw15840\paperh12240\widowctrl\ftnbj \fet0\sectd \lndscpsxn\linex0 '  ;
                %end ;
            %end ;
        %else %do ;
            %if %upcase(%substr(&paper,1,1)) = LEGAL %then %do ;
               put '\paperw12240\paperh20160\widowctrl\ftnbj \fet0\sectd \linex0 '  ;
               %end ;
            %else %do ;
               put '\paperw12240\paperh15840\widowctrl\ftnbj \fet0\sectd \linex0 '  ;
               %end ;
            %end ;
        %if &leftm eq %then %do ;
            put "\margl720" ;
            %end ;
        %else %do ;
            _value = "\margl" || compress(put(1440*&leftm,5.)) ;
            put _value ;
            %end ;
        %if &rightm eq %then %do ;
            put "\margr720" ;
            %end ;
        %else %do ;
            _value = "\margr" || compress(put(1440*&rightm,5.)) ;
            put _value ;
            %end ;
        %if &topm eq %then %do ;
            put "\margt720" ;
            %end ;
        %else %do ;
            _value = "\margt" || compress(put(1440*&topm,5.)) ;
            put _value ;
            %end ;
        %if &bottom eq %then %do ;
            put "\margb720" ;
            %end ;
        %else %do ;
            _value = "\margb" || compress(put(1440*&bottom,5.)) ;
            put _value ;
            %end ;
        run ;

%mend rtfprofi ;


