/*
 * %rtfprofo      Copyright Stat-Tech Services, LLC  1999
 *
 *       For questions or comments please contact:
 *
 *       Stat-Tech Services, LLC
 *       105 N. Crabtree Knoll
 *       Chapel Hill, NC  27514
 *       Fax:  919-493-2776 / Phone:  919-493-2106
 *
 *       Questions - please e-mail stattsrv@aol.com
 *
 *       see http://www.stattechservices.com/rtf.htm
 * 
 * Purpose-  creates an RTF quick table
 * Author-   G. DeMuth
 * Date-     28 Feb 1998
 * 
 * Modifications
 *
 */

%macro rtfprofo ;

    %local _i _j _ce1 _ce2 _ce3 _ce4 _ce5 _ce6 _ce7
           _ce8 _ce9 _ce10 _ce11 _ce12 _ce13 _ce14
           _ce15 _ce16 _ce17 _ce18 _ce19 _ce20 _celag
           _mlvl _txloc _txlast _txwid ;

    %* Close off any unclosed spanning headers - ged001 ;
    %do _i = 1 %to &_clevel ;
        %rtfspend ;
        %end ;

    %* reverse level numbering ;
    %let _mlvl = 0 ;
    %do _i = 1 %to &_nspan ;
        %if &&_slvl&_i > &_mlvl %then %do ;
            %let _mlvl = &&_slvl&_i ;
            %end ;
        %end ;
    %do _i = 1 %to &_nspan ;
        %let _slvl&_i = %eval(&_mlvl - &&_slvl&_i + 1) ;
        %end ;

    %* Calculate column endpoints ;
    %let _ce1 = &_wid1 ;
    %let _celag = &_ce1 ;
    %do _i = 2 %to &_ncol ;
        %let _ce&_i = %eval(&_celag + &&_wid&_i) ;
        %let _celag = &&_ce&_i ;
        %end ;

    %* Determine header heights for columns ;
    data _null_ ;
        length _lab _labtmp $200 ;
        _maxht = 1 ;
        %do _i = 1 %to &_ncol ;
            _lab = trim("&&_lab&_i") ;
            _lablen = length(_lab) ;
            _labtmp = repeat(' ',199) ;
            _tmploc = 0 ;
            _colht = 1 ;
            do _i = 1 to _lablen ;
                _tmploc = _tmploc + 1 ;
                c = substr(_lab,_i,1) ;
                if c = "&_split" then do ;
                    _colht = _colht + 1 ;
                    substr(_labtmp,_tmploc,4) = '\par ' ;
                    _tmploc = _tmploc + 4 ;
                    end ;
                else do ;
                    substr(_labtmp,_tmploc,1) = c ;
                    end ;
                end ;
            call symput("_lab&_i",trim(_labtmp)) ;
            call symput("_colht&_i",compress(put(_colht,2.))) ;
            if _maxht < _colht then _maxht = _colht ;
            %end ;
        call symput('_maxht',compress(put(_maxht,2.))) ;
        run ;

    %* Obtain all sorting variables ;
    %let _sortc = 0 ;
    %do _i = 1 %to &_ncol ;
        %if &&_typ&_i = BY %then %do ;
            %let _sortc = %eval(&_sortc + 1) ;
            %end ;
        %end ;

    %* Sort the data ;
    %if &_sortc > 0 or &_line ne %then %do ;
        proc sort data=&_data ;
            by &_line %do _i = 1 %to &_sortc ;
                                &&_var&_i
                             %end ;
                           ;
            run ;
        %end ;

    %* Map spanning column names to the order of columns ;
    %do _i = 1 %to &_nspan ;
        %do _j = 1 %to &_ncol ;
            %if %upcase(&&_strt&_i) = %upcase(&&_var&_j) %then %do ;
                %let _strt&_i = &_j ;
                %end ;
            %if %upcase(&&_stop&_i) = %upcase(&&_var&_j) %then %do ;
                %let _stop&_i = &_j ;
                %end ;
            %end ;
        %end ;

    %* print RTF file ;
    data _null_ ;
        length _value $200 ;
        retain line pline 0 ;
        retain pagtop 1 ;
        file "&_fname" noprint notitles mod ;
        set &_data end=eof ;

        %if &_sortc > 0 or &_line ne %then %do ;
            by &_line %do _i = 1 %to &_sortc ;
                                &&_var&_i
                             %end ;
                           ;
            %end ;

        %* Set up RTF header ;
        if line = 0 then link headers ;

        %* Moved from behind following text 1/14/1999 ;
        line = line + 1 ;
        pline = pline + 1 ;

        if eof 		
            %if &_sortc > 0 or &_line ne and &_style ne BOX &_style ne LINEXB %then %do ;
                or ( %if &_sortc > 0 %then %do ;
                        last.&&_var&_sortc
                        %end ;
                    %else %do ;
                        last.&_line
                        %end ;
                    )  
               %end ;
            %else %if &_sortc > 0 or &_line ne %then %do ;
                or ( %if &_sortc > 0 %then %do ;
                        last.&&_var&_sortc
                        %end ;
                    %else %do ;
                        last.&_line
                        %end ;
                     ) 
                %end ;
            then do ;
            put "\trowd\tgraph&_linespc\trqc"  ;
            %do _i = 1 %to &_ncol ;
                %if &_style = OUTLINE and &_i = 1 %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    %end ;
                %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = BOX %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                put "\clbrdrb\brdrs\brdrw15\cellx&&_ce&_i" ;
                %end ;
            put "\pard" ;
            %do _i = 1 %to &_ncol ;
                %if &_i <= &_sortc %then %do ;
                        /*jri001*/
                    %if &_uleft = Y %then %do ;
                        if first.&&_var&_i or pagtop = 1 then _value = trim(left(put(&&_var&_i,&&_fmt&_i)))  ;
                        %end ;
                    %else %do ;
                        if first.&&_var&_i or pagtop = 1 then _value = trim(put(&&_var&_i,&&_fmt&_i))  ;
                        %end ;
                    else _value = "" ;
                    %end ;
                %else %do ;
                        /*jri001*/
                    %if &_uleft = Y %then %do ;
                        _value = trim ( left( put(&&_var&_i,&&_fmt&_i))) ;
                        %end ;
                    %else %do ;
                        _value = trim ( put(&&_var&_i,&&_fmt&_i)) ;
                        %end ;
                    %end ;
                %if &&_just&_i ne d and &&_just&_i ne m %then %do ;
                    %if &&_hang&_i eq 0 %then %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %else %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi-&&_hang&_i\li&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %end ;
                %else %if &&_just&_i eq d %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys\tqdec\tx&_txloc " @@ ;
                    %end ;
                %else %if &&_just&_i eq m %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    %end ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@;
                put "\cell\pard" ;
                %end ;
            put "\widctlpar\intbl \row \pard" ;
            put ;
            pagtop = 0 ;  %* Reset pagtop 1/14/1999 ;
            end ;
        else do ;
            put "\trowd\tgraph&_linespc\trqc"  ;
            _lcond = 0 ;
            %do _i = 1 %to &_ncol ;
                %if &_style = OUTLINE and &_i = 1 %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    %end ;
                %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = LINEX %then %do ;
                    %if &_sortc > 0 %then %do ;
                        if last.&&_var&_i or _lcond = 1 then do ;
                            _lcond = 1 ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %else %if &_line ne %then %do ;
                        if last.&_line then do ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %end ;
                %if &_style = BOX %then %do ;
                    %if &_i <= &_sortc %then %do ;
                        if last.&&_var&_i or _lcond = 1 then do ;
                            _lcond = 1 ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %if &_i > &_sortc %then %do ;
                        if _lcond = 1 then do ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    %if &_sortc = 0 and &_line ne %then %do ;
                        if last.&_line then do ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            end ;
                        %end ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                put "\cellx&&_ce&_i" ;
                %end ;
            put "\pard" ;
            %do _i = 1 %to &_ncol ;
                %if &_i <= &_sortc %then %do ;
                        /*jri001*/
                    %if &_uleft = Y %then %do ;
                        if first.&&_var&_i or pagtop = 1 then _value = trim( left( put(&&_var&_i,&&_fmt&_i)))  ;
                        %end ;
                    %else %do ;
                        if first.&&_var&_i or pagtop = 1 then _value = trim( put(&&_var&_i,&&_fmt&_i))  ;
                        %end ;
                    else _value = "" ;
                    %end ;
                %else %do ;
                        /*jri001*/
                    %if &_uleft = Y %then %do ;
                        _value = trim( left( put(&&_var&_i,&&_fmt&_i))) ;
                        %end ;
                    %else %do ;
                        _value = trim( put(&&_var&_i,&&_fmt&_i)) ;
                        %end ;
                    %end ;
                %if &&_just&_i ne d and &&_just&_i ne m %then %do ;
                    %if &&_hang&_i eq 0 %then %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %else %do ;
                        put "\q&&_just&_i\widctlpar\intbl\fi-&&_hang&_i\li&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                        %end ;
                    %end ;
                %else %if &&_just&_i eq d %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys\tqdec\tx&_txloc " @@ ;
                    %end ;
                %else %if &&_just&_i eq m %then %do ;
                    %if &_i = 1 %then %do ;
                        %let _txloc = %eval(&&_ce&_i / 2) ;
                        %end ;
                    %else %do ;
                        %let _txlast = %eval(&_i - 1) ;
                        %let _txwid = %eval(&&_ce&_i - &&_ce&_txlast) ;
                        %let _txloc = %eval(&_txwid / 2) ;
                        %end ;
                    put "\ql\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    %end ;
                _zl = length(_value) ;
                put _value $varying200. _zl ;
                put "\cell\pard" ;
                %end ;
            put "\widctlpar\intbl \row \pard" ;
            pagtop = 0 ;
            put ;
            end ;

        %* Blank lines between BY variables ;
        %if (&_sortc > 0 or &_line ne) and &_style ne BOX and &_style ne LINEX %then %do ;
            if
                %if &_line ne and &_sortc = 0 %then %do ;
                    (last.&_line and not eof)
                    %end ;
                %else %if &_line eq and &_sortc > 0 %then %do ;
                    (last.&&_var&_sortc and not eof)
                    %end ;
                %else %if &_line ne and &_sortc > 0 %then %do ;
                    ((last.&_line or last.&&_var&_sortc) and not eof)
                    %end ;
                   then do ;
                pline = pline + 1 ;
                put "\trowd\tgraph&_linespc\trqc"  ;
                %do _i = 1 %to &_ncol ;
                    %if &_style = OUTLINE and &_i = 1 %then %do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        %end ;
                    %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        %end ;
                    %if &_style = LINEXB %then %do ;
                        put "\clbrdrt\brdrs\brdrw15" @@ ;
                        %end ;
                    %* GED 1/11/1999 fix to bottom border ;
                    /****
                    if (pline + 1) >= &_tabline or (pline + _next + 1) > &_tabline then do ;   
                        put "\clbrdrb\brdrs\brdrw15" @@ ;
                        end ;
                     ***/
                    put "\cellx&&_ce&_i" ;
                    %end ;
                put "\pard" ;
                %do _i = 1 %to &_ncol ;
                    _value = "\cell\pard" ;
                    put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    put _value  ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;
                put ;
                end ;
            %end ;

        return ;

        %* Headers ;
        headers:
            pagtop = 1 ;
            put "\qc\widctlpar\fi0\f1\fs2 \par" ;

            %* Spanning headers ;
            %* Get maximum depth of spanning headers ;
            %let _depth = 0 ;
            %do _i = 1 %to &_nspan ;
                %if &&_slvl&_i > &_depth %then %do ;
                    %let _depth = &&_slvl&_i ;
                    %end ;
                %end ;

            %do _i = 1 %to &_depth ;
                %let _c = %eval(&_depth - &_i + 1) ;
                %* Spanning headers are assumed ordered - pick them out to create the line ;
                %* For each spanning header put blank cells as necessary ;
                %let _tots = 0 ;
                %do _j = 1 %to &_nspan ;
                    %if &&_slvl&_j = &_c %then %do;
                        %let _tots = %eval(&_tots + 1) ;
                        %end ;
                    %end ;
                %* %put _TOTS &_tots ;

                %* First make cell definitions ;
                %let _currcol = 0 ;
                %let _lagcnt = 0 ;
                put "\trowd\tgraph&_linespc\trqc"  ;
                %do _j = 1 %to &_nspan ;
                    %* Find if spanning column matches current depth ;
                    %if &&_slvl&_j = &_c %then %do ;
                        %let _lagcnt = %eval(&_lagcnt + 1) ;
                        %let _currend = &&_stop&_j ;
                        %* Line information ;
                        %if &_style = OUTLINE and &_currcol = 0 %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            %end ;
                        %* %if &_style = OUTLINE and &_lagcnt = &_tots %then %do ;
                            %* put "\clbrdrr\brdrs\brdrw15" @@ ;
                            %* %end ;
                        %if &_style = BOX %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_i = 1 %then %do ;
                            put "\clbrdrt\brdrs\brdrw15" @@ ;
                            %end ;
                        %* Find if need adjust for column location ;
                        %if &_currcol ne %eval(&&_strt&_j - 1) %then %do ;
                            %* Fix currcol by making blank cell number equal to stop column ;
                            %let _currcol = %eval(&&_strt&_j - 1) ;
                            put "\cellx&&_ce&_currcol" ;
                            %end ;
                        %let _currcol = &&_stop&_j ;
                        %put _currcol = &_currcol   _lagend = &_lagcnt   ;
                        %* Make cell with borders as needed ;
                        %if &_style = OUTLINE and &_currcol = 0 %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_style = OUTLINE and &_currend = &_ncol %then %do ;  %* change to cell 12/16/98 ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_style = BOX %then %do ;
                            put "\clbrdrl\brdrs\brdrw15" @@ ;
                            put "\clbrdrr\brdrs\brdrw15" @@ ;
                            put "\clbrdrb\brdrs\brdrw15" @@ ;
                            %end ;
                        %if &_i = 1 %then %do ;
                            put "\clbrdrt\brdrs\brdrw15" @@ ;
                            %end ;
                        %let _tmpc = &&_stop&_j ;
                        put "\clbrdrb\brdrs\brdrw15\cellx&&_ce&_tmpc" ;
                        %end ;
                    %end ;
                %* fix for currend condition - add extra cell ;
                %if &_currend ne &_ncol %then %do ;
                    %if &_style = OUTLINE %then %do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        %end ;
                    %if &_style = BOX %then %do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        put "\clbrdrb\brdrs\brdrw15" @@ ;
                        %end ;
                    %if &_i = 1 %then %do ;
                        put "\clbrdrt\brdrs\brdrw15" @@ ;
                        %end ;
                    %let _tmpc = &&_stop&_j ;
                    put "\cellx&&_ce&_ncol" ;
                    %end ;
                put "\pard" ;

                %* Write entries for each cell ;
                %let _currcol = 0 ;
                %do _j = 1 %to &_nspan ;
                    %* Find if spanning column matches current depth ;
                    %if &&_slvl&_j = &_c %then %do ;
                        %let _currend = &&_stop&_j ;
                        %* Find if need adjust for column location ;
                        %if &_currcol ne %eval(&&_strt&_j - 1) %then %do ;
                            %* Fix currcol by making blank cell number equal to stop column ;
                            %let _currcol = %eval(&&_strt&_j - 1) ;
                            put "\qc\widctlpar\intbl\fi0\f&_labf\fs&_labs \cell\pard" ;
                            %end ;
                        %* Make cell with borders as needed ;
                        put "\qc\widctlpar\intbl\fi0\f&_labf\fs&_labs " @@ ;
                        %if &_labb = Y %then %do ;
                            put "\b " @@ ;
                            %end ;
                        %if &_labu = Y %then %do ;
                            put "\ul " @@ ;
                            %end ;
                        %if &_labi = Y %then %do ;
                            put "\i " @@ ;
                            %end ;
                        _value = trim("&&_stxt&_j") ;
                        _zl = length(_value) ;
                        put _value $varying200. _zl @@ ;
                        %if &_labb = Y %then %do ;
                            put "\b0 " @@ ;
                            %end ;
                        %if &_labu = Y %then %do ;
                            put "\ul0 " @@ ;
                            %end ;
                        %if &_labi = Y %then %do ;
                            put "\i0 " @@ ;
                            %end ;
                        put "\cell\pard" ;
                        %let _currcol = &&_stop&_j ;
                        %end ;
                    %end ;
                %if &_currend ne &_ncol %then %do ;
                    put "\qc\widctlpar\intbl\fi0\f&_labf\fs&_labs \cell\pard" ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;
                   
                %end ;


            %* Table headers ;
            put "\trowd\tgraph&_linespc\trqc"  ;
            %do _i = 1 %to &_ncol ;
                %if &_nspan < 1 %then %do ;
                    put "\clbrdrt\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = OUTLINE and &_i = 1 %then %do ;
                    %let _lagcol = &_currcol ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    %end ;
                %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                %if &_style = BOX %then %do ;
                    put "\clbrdrl\brdrs\brdrw15" @@ ;
                    put "\clbrdrr\brdrs\brdrw15" @@ ;
                    %end ;
                put "\clbrdrb\brdrs\brdrw15\cellx&&_ce&_i" ;
                %end ;
            put "\pard" ;
            %do _i = 1 %to &_ncol ;
                %if &&_just&_i = d or &&_just&_i = m %then %do ;
                    put "\qc\widctlpar\intbl\fi&&_hang&_i\f&_labf\fs&_labs " @@ ;
                    %end ;
                %else %do ;
                    put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_labf\fs&_labs " @@ ;
                    %end ;
                %if &_labb = Y %then %do ;
                    put "\b " @@ ;
                    %end ;
                %if &_labu = Y %then %do ;
                    put "\ul " @@ ;
                    %end ;
                %if &_labi = Y %then %do ;
                    put "\i " @@ ;
                    %end ;
                %let _ch = &&_colht&_i ;
                %if &_ch < &_maxht %then %do ;
                    %let _c = %eval(&_maxht - &_ch) ;
                    %do _j = 1 %to &_c ;
                        put '\par ' @@ ;
                        %end ;
                    %end ;
                _value = trim("&&_lab&_i") ;
                _zl = length(_value) ;
                put _value $varying200. _zl @@ ;
                %if &_labb = Y %then %do ;
                    put "\b0 " @@ ;
                    %end ;
                %if &_labu = Y %then %do ;
                    put "\ul0 " @@ ;
                    %end ;
                %if &_labi = Y %then %do ;
                    put "\i0 " @@ ;
                    %end ;
                put "\cell\pard" ;
                %end ;
            put "\widctlpar\intbl \row \pard" ;
            
        %if &_hedskip=Y %then %do;
        
                pline = pline + 1 ;
                put "\trowd\tgraph&_linespc\trqc"  ;
                %do _i = 1 %to &_ncol ;
                    %if &_style = OUTLINE and &_i = 1 %then %do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        %end ;
                    %else %if &_style = OUTLINE and &_i = &_ncol %then %do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        %end ;
                    put "\cellx&&_ce&_i" ;
                    %end ;
                put "\pard" ;
                %do _i = 1 %to &_ncol ;
                    _value = "\cell\pard" ;
                    put "\q&&_just&_i\widctlpar\intbl\fi&&_hang&_i\f&_bodyf\fs&_bodys " @@ ;
                    put _value  ;
                    %end ;
                put "\widctlpar\intbl \row \pard" ;

        %end;
                                         
       return ;

       run ;

%mend rtfprofo ;


