/*
 * %rtfprofp   Copyright Stat-Tech Services, LLC, 1999
 *
 *       For questions or comments please contact:
 *
 *       Stat-Tech Services, LLC
 *       105 N. Crabtree Knoll
 *       Chapel Hill, NC  27514
 *       Fax:  919-493-2776 / Phone:  919-493-2106
 *
 *       Questions - please e-mail stattsrv@aol.com
 *
 *       see http://www.stattechservices.com/rtf.htm
 * 
 * Purpose-  Puts text into to an RTF paragraph via a variety of approaches:
 *
 *           1) As a normal text paragraph OR as a table with up to 5 cells
 *           2) Paragraph formatting/text options are available on individual paragraphs
 *              and within table cells.
 *           3) If both text and a variable are entered, then issue text followed by
 *              the variable.
 *           4) Programmer controls the width of table cells.
 *           5) Each cell can be specified as text OR as input from a file.
 *           6) For input from a file, records can be separated by a carriage return 
 *              or with blanks.
 *
 * Author-   G. DeMuth
 * Date-     28 Feb 1999
 * 
 * Modifications
 *
 */

%macro rtfproft(width1=,font1=3,fonts1=10,text1=,bold1=N,ital1=N,under1=N,data1=,var1=,fmt1=,break1=Y,find1=0,lmarg1=0,jst1=C,
                width2=,font2=3,fonts2=10,text2=,bold2=N,ital2=N,under2=N,data2=,var2=,fmt2=,break2=Y,find2=0,lmarg2=0,jst2=C,
                width3=,font3=3,fonts3=10,text3=,bold3=N,ital3=N,under3=N,data3=,var3=,fmt3=,break3=Y,find3=0,lmarg3=0,jst3=C,
                width4=,font4=3,fonts4=10,text4=,bold4=N,ital4=N,under4=N,data4=,var4=,fmt4=,break4=Y,find4=0,lmarg4=0,jst4=C,
                width5=,font5=3,fonts5=10,text5=,bold5=N,ital5=N,under5=N,data5=,var5=,fmt5=,break5=Y,find5=0,lmarg5=0,jst5=C,
                btop=N,bleft=N,bright=N,bbot=N,grid=N) ; 

    %local _i _j _k _allw _tott ;

    %let _tott = 0 ;
    %let _allw = 1 ;

    %* Count number of actual entries ;
    %do _i = 1 %to 5 ;
        %if %bquote(&&text&_i) ne or %bquote(&&var&_i) ne %then %do ;
            %let _tott = &_i ;
            %if %bquote(&&width&_i) eq %then %do ;
                %let _allw = 0 ;
                %end ;
            %end ;
        %end ;

    %* Create table cell definitions if widths are all non-missing ;
    %if &_allw = 1 %then %do ;

        data _null_ ;
            length _value $200 ;
            file "&_fname" noprint notitles mod ;
            array widths {*} w1-w5 ;
            btop = upcase(substr("&btop",1,1)) ;
            bleft = upcase(substr("&bleft",1,1)) ;
            bright = upcase(substr("&bright",1,1)) ;
            bbot = upcase(substr("&bbot",1,1)) ;
            grid = upcase(substr("&grid",1,1)) ;
            %if &_allw = 1 %then %do ;
                %do _i = 1 %to &_tott ;
                    w&_i = 1440*&&width&_i ;
                    %end ;
                %end ;
            put "\trowd\tgraph&_linespc\trqc"  ;
            totw = 0 ;
            do _i = 1 to &_tott ;
                totw = totw + widths{_i} ;
                if btop = 'Y' then do ;
                    put "\clbrdrt\brdrs\brdrw15" @@ ;
                    end ;
                if bbot = 'Y' then do ;
                    put "\clbrdrb\brdrs\brdrw15" @@ ;
                    end ;
                if bleft = 'Y' or grid = 'Y' then do ;
                    if (_i = 1 and bleft = 'Y') or (_i > 1 and grid = 'Y') then do ;
                        put "\clbrdrl\brdrs\brdrw15" @@ ;
                        end ;
                    end ;
                if bright = 'Y' or grid = 'Y' then do ;
                    if (_i = &_tott and bright = 'Y') or (_i < &_tott and grid = 'Y') then do ;
                        put "\clbrdrr\brdrs\brdrw15" @@ ;
                        end ;
                    end ;
                _value = trim("\cellx" || compress(put(totw,5.))) ;
                put _value ;
                end ;
            run ;

        %end ;

    %* Print paragraphs or cell contents. ;
    %do _i = 1 %to &_tott ;

        %* Set up the row or table cell ;
        data _null_ ;
            file "&_fname" noprint notitles mod ;
            put "\pard" ;
            run ;

        %* Print the text and/or variable ;
        data _null_ ;
            file "&_fname" noprint notitles mod ;
            %* justification ;
            put "\q%lowcase(%substr(&&jst&_i,1,1))" @@ ;
            %* table information ;
            %if &_allw = 1 %then %do ;
                put "\widctlpar\intbl" @@ ;
                %end ;
            %* set indents ;
            length lm fi $20. ;
            lm = compress(put(1440*&&lmarg&_i,5.)) ;
            fi = compress(put(1440*&&find&_i,5.)) ;
            put "\li" lm "\fi" fi @@ ;
            %* fonts ;
            put "\f&&font&_i..\fs%eval(2*&&fonts&_i) " @@ ;
            %if %upcase(%substr(&&bold&_i,1,1)) = Y %then %do ;
                put "\b " @@ ;
                %end ;
            %if %upcase(%substr(&&ital&_i,1,1)) = Y %then %do ;
                put "\i " @@ ;
                %end ;
            %if %upcase(%substr(&&under&_i,1,1)) = Y %then %do ;
                put "\ul " @@ ;
                %end ;
            %* put text if present ;
            %if %bquote(&&text&_i) ne %then %do ;
                _value = trim("&&text&_i") ;
                put _value ;
                %if %upcase(%substr(&&break&_i,1,1)) = Y and (&&var&_i ne or &_allw = 0) %then %do ;
                    put "\par" ;
                    %end ;
                %end ;
            run ;
            
        %* If variable is specified print the variable ;
        %if &&var&_i ne %then %do ;
        
            data _null_ ;
                set &&data&_i end=eof ;
                file "&_fname" noprint notitles mod ;
                length _value $200. ;
                _value = trim(put(&&var&_i,&&fmt&_i)) ;
                put _value ;
                %if %upcase(%substr(&&break&_i,1,1)) = Y %then %do ;
                    put "\par" ;
                    %end ;
                %else %if %upcase(%substr(&&break&_i,1,1)) = C %then %do ;
                    if not eof then put ", " ;
                    if eof then put '\par' ;
                    %end ;
                %else %do ;
                    if not eof then put " " ;
                    if eof then put '\par' ;
                    %end ;
                run ;
                
            %end ;

        data _null_ ;
            file "&_fname" noprint notitles mod ;
            %if %upcase(%substr(&&bold&_i,1,1)) = Y %then %do ;
                put "\b0" @@ ;
                %end ;
            %if %upcase(%substr(&&ital&_i,1,1)) = Y %then %do ;
                put "\i0" @@ ;
                %end ;
            %if %upcase(%substr(&&under&_i,1,1)) = Y %then %do ;
                put "\ul0" @@ ;
                %end ;
            %if &_allw = 1 %then %do ;
                put "\cell\pard" ;
                %end ;
            run ;
        
        %end ;

       %if &_allw eq 1 %then %do ;
            data _null_ ;
                file "&_fname" noprint notitles mod ;
                put "\row" ;
            %end ;


%mend rtfproft ;


