/* rtfttlgn.mac

    Copyrighted by Stat-Tech Services, LLC   1998

    Macro Family:  Title Management System (specialized for rtf subsystem)

    Purpose:    Obtains a title from the titles data set.

                NOTE:  This macro should be customized for every project.
                
                NOTE:   This macro introduces a new footnote tyype into the title data
                        model.  The attached or tail type footnote is justified  with the 
                        table.  It is identified with the instruction:
                        
                        A#= where A indicates footnote is attached and # indicates line number.

    Required Macros:  _qtslash (see macro body for example of how _qtslash is used to quote backslashes).

    Parameters:

        Name                    Definition
        -------------------------------------------------------------------------------------
        progname                Program reference for the title.
        ref                     Reference code within the progname for the actual title.
        printto=OFF             (or ON) tells whether to issue a proc printto command
                                mapping the output to the designated file.

        PRINTTO=ON              creates a global variable with the program name stored.

        filedest=               Used to specify the directory for the output file to run.
                                Example:  filedest=\project\sasout  (not \project\sasout\)

        titlfile=TITLE.TXT      text file containing title information.
        tfont=3                 Title font number (see documentation for rtfinit).
        tfsize=12               Title font size.
        tlmarg=15               Size, in tenths of an inch, of the left field in the three field
                                title line (allows multiple justifications in a single title line).
        trmarg=15               Size, in tenths of an inch, of the right field in the three field
                                title line.
        ffont=3                 Footnote font number.
        ffsize=10               Footnote font size.
        flmarg=15               Size, in tenths of an inch, of the left field in the three field
                                footnote line (allows multiple justifications in a single footnote line).
        frmarg=15               Size, in tenths of an inch, of the right field in the three field
                                footnote line.

    Example:  %rtftitlgn(list001,01,pp=Y) ;

        ---------------------------------------------------------------------------------

        jri 12/3/98 6:04PM

        Revisions:  

*/

%macro rtfttlgn(progname,ref,printto=OFF,titlfile=TITLE.TXT,filedest=, 
        tfont=3, tfsize=11, tbold=y, tlmarg=0, trmarg=0, ffont=3, ffsize=10, flmarg=15, frmarg=15, f9=4) ;

		%* Create global footnote and titel variables and set them to null ;
		%global _totfoot _tottail
		
		        _fltxt1 _fltxt2 _fltxt3 _fltxt4 _fltxt5
		        _fltxt6 _fltxt7 _fltxt8 _fltxt9 _fltxt10
		        _fctxt1 _fctxt2 _fctxt3 _fctxt4 _fctxt5
		        _fctxt6 _fctxt7 _fctxt8 _fctxt9 _fctxt10
		        _frtxt1 _frtxt2 _frtxt3 _frtxt4 _frtxt5
		        _frtxt6 _frtxt7 _frtxt8 _frtxt9 _frtxt10 

		        _atxt1 _atxt2 _atxt3 _atxt4 _atxt5
		        _atxt6 _atxt7 _atxt8 _atxt9 _atxt10
		
		        _tltxt1 _tltxt2 _tltxt3 _tltxt4 _tltxt5
		        _tltxt6 _tltxt7 _tltxt8 _tltxt9 _tltxt10
		        _tctxt1 _tctxt2 _tctxt3 _tctxt4 _tctxt5
		        _tctxt6 _tctxt7 _tctxt8 _tctxt9 _tctxt10
		        _trtxt1 _trtxt2 _trtxt3 _trtxt4 _trtxt5
		        _trtxt6 _trtxt7 _trtxt8 _trtxt9 _trtxt10 
		        
		        ;

		%let _totfoot = 0 ;
		%let _tottail = 0 ;
		%do _i = 1 %to 10 ;
		    %let _fltxt&_i = ;
		    %let _fctxt&_i = ;
		    %let _frtxt&_i = ;
		    
		    %let _atxt&_i = ;
		    
		    %let _tltxt&_i = ;
		    %let _tctxt&_i = ;
		    %let _trtxt&_i = ;
		    %end ;
       
        %*** capture linesize ;
        %let _ls = %sysfunc(getoption(ls)) ;

        %*** Obtain the title lines from the source ;
        data _null_ ;
        
            length ltxt1-ltxt10 rtxt1-rtxt10 ctxt1-ctxt10 $200. ;
            retain ltxt1-ltxt10 rtxt1-rtxt10 ctxt1-ctxt10 ' ' ;
            length lftxt1-lftxt10 rftxt1-rftxt10 cftxt1-cftxt10 $200. ;
            retain lftxt1-lftxt10 rftxt1-rftxt10 cftxt1-cftxt10 ' ' ;
            length atxt1-atxt10 $200. ;
            retain atxt1-atxt10 ' ' ;

            array lefts {*} ltxt1-ltxt10 ;
            array centers {*} ctxt1-ctxt10 ;
            array rights {*} rtxt1-rtxt10 ;
            array flefts {*} lftxt1-lftxt10 ;
            array fcenters {*} cftxt1-cftxt10 ;
            array frights {*} rftxt1-rftxt10 ;
            array alefts {*} atxt1-atxt10 ;

%* ENTER STANDARD HEADERS HERE: ;
            
            %*** Set up custom headers here ;
            %*** use ltxt1-ltxt10 for all standard left justified headers ;
            %*** use ctxt1-ctxt10 for all standard center justified headers ;
            %*** use rtxt1-rtxt10 for all standard right justified headers ;
            %*** ;
            %*** The following are provided as example of a standard header on ;
            %*** title line 1 ;
            %*** ltxt1= "&sysdate : &systime" ;
            %*** ctxt1= "PROJECT XXX" ;
            %*** rtxt1= "PAGE ### OF ###" ;
            %***atxt&f9="&sysdate &systime "||'\par '||"%_qtslash( invar=%bquote(&__projec &_prognam &__tabcod))";

                /* Italicized */
            atxt&f9="\i &sysdate &systime "||'\par '||"%_qtslash( invar=%bquote(&__projec &_prognam &__tabcod))"||" \i0 ";

%* END STANDARD HEADERS ;

            %*** This text captures standard headers from the title data file ;
            length cprog cref  $200 ;
            retain cprog cref ' ' ;

            infile "&titlfile" length=ll end=eof ;  %* read into the title lines ;
            input infline $varying200. ll ;

            %*** decode input line instructions ;
            length instruct $80 ;


            %*** get instruction ;
            instruct = upcase(scan(infline,1,'=')) ;

            %*** change current reference program ;
            if instruct = 'PROGNAME' then do ;
                cprog = scan(infline,2,'=') ;
                end ;
            %*** change current reference within program name ;
            if upcase(cprog) = upcase("&progname") and instruct = 'REF' then do ;
                infline=resolve( infline);
                cref = scan(infline,2,'=') ;
                end ;

            %*** look for titles if current program and reference match call parameters ;
            if upcase(cprog) = upcase("&progname") and upcase(cref) = upcase("&ref") then do ;
                if substr(instruct,1,1) = 'T' and substr(instruct,2,1) in ('1' '2' '3' '4' '5' '6' '7' '8' '9')
                    then do ;
                        tnum = input(scan(instruct,1,'tclrTCLR'),2.) ;
                        if 0 < tnum < 10 then do ;
                            tjust = scan(instruct,2,'0123456789') ;
                            if tjust = 'C' or tjust = ' ' then centers{tnum} = scan(infline,2,'=') ;
                            /*
                            else if tjust = 'L' then do ;
                                tloc = index(infline,'=') ;
                                tlen = length(infline) ;
                                lefts{tnum} = trim(put(substr(infline,tloc+1,tlen-tloc),$&_ls..)) ;
                                * put lefts{tnum} $&_ls.. ;
                                end ;
                                */
                            else if tjust = 'R' then rights{tnum} = scan(infline,2,'=') ;
                            if tjust in ('L' 'R') then do;
                                tmarg=input(scan(instruct,2,'fclrFCLR'),2.);
                                if tmarg>. then call symput( 't'||trim(left(tjust))||'marg', trim(left(tmarg)) );
                            end ;
                        end ;
                end;

                /*regular footnotes.
                */
                if substr(instruct,1,1) = 'F' and substr(instruct,2,1) in ('1' '2' '3' '4' '5' '6' '7' '8' '9')
                    then do ;
                        tnum = input(scan(instruct,1,'fclrFCLR'),2.) ;
                        if 0 < tnum < 10 then do ;
                            tjust = scan(instruct,2,'0123456789') ;
                            if tjust = 'L' or tjust = ' ' then do ;
                                tloc = index(infline,'=') ;
                                tlen = length(infline) ;
                                flefts{tnum} = trim(put(substr(infline,tloc+1,tlen-tloc),$&_ls..)) ;
                                * put flefts{tnum} $&_ls.. ;
                                end ;
                            else if tjust = 'C' then fcenters{tnum} = scan(infline,2,'=') ;
                            else if tjust = 'R' then frights{tnum} = scan(infline,2,'=') ;
                            if tjust in ('L' 'R') then do;
                                tmarg=input(scan(instruct,2,'fclrFCLR'),2.);
                                if tmarg>. then call symput( 'f'||trim(left(tjust))||'marg', trim(left(tmarg)) );
                            end ;
                        end ;
                end;


                /* Tail style footnotes.
                */
                if substr(instruct,1,1) = 'A' and substr(instruct,2,1) in ('1' '2' '3' '4' '5' '6' '7' '8' '9')
                    then do ;
                        tnum = input(scan(instruct,1,'afclrAFCLR'),2.) ;
                        if 0 < tnum < 10 then do ;
                            tjust = scan(instruct,2,'0123456789') ;
                            if tjust = 'L' or tjust = ' ' then do ;
                                tloc = index(infline,'=') ;
                                tlen = length(infline) ;
                                alefts{tnum} = trim(put(substr(infline,tloc+1,tlen-tloc),$200.)) ;
                                *alefts{tnum} = trim(put(substr(infline,tloc+1,tlen-tloc),$&_ls..)) ;
                                end ;
                            else if tjust in ('C'  'R') then do;
                                %put WARNING: No center or right justification for tail-type foonotes.;
                            end ;
                        end ;
                 end ;  
            end;

            %*** if end of file build macro variables with title information ;
            if eof then do ;
                maxh = 0 ;
                maxf = 0 ;
                maxa = 0 ;
                %*** headers ;
                do i=1 to 10 ;
                    length ctitle $&_ls ;
                    ctitle = repeat(' ',&_ls - 1) ;
                    %*** store left justified text ;
                    if lefts{i} ne ' ' then do ;
                        maxh = i ;
                        call symput( '_tltxt'||trim( left( i)), trim( left(lefts{i})));
                        end ;
                    %*** store right justified text ;
                    if rights{i} ne ' ' then do ;
                        maxh = i ;
                        call symput( '_trtxt'||trim( left( i)), trim( left(rights{i})));
                        end ;
                    %*** store center justified text ;
                    if centers{i} ne ' ' then do ;
                        maxh = i ;
                        call symput( '_tctxt'||trim( left( i)), trim( left(centers{i})));
                        end ;
                    end ;

                %*** regular footers ;
                do i=1 to 10 ;
                    %*** store left justified text ;
                    if flefts{i} ne ' ' then do ;
                        maxf = i ;
                        call symput('_fltxt' || compress(i),trim(flefts{i})) ;
                        end ;
                    %*** store right justified text ;
                    if frights{i} ne ' ' then do ;
                        maxf = i ;
                        call symput('_frtxt' || compress(i),trim(frights{i})) ;
                        end ;
                    %*** store center justified text ;
                    if fcenters{i} ne ' ' then do ;
                        maxf = i ;
                        call symput('_fctxt' || compress(i),trim(fcenters{i})) ;
                        end ;
                    end ;

                %*** tail-type footers ;
                do i=1 to 10 ;
                    %*** store left justified text ;
                    if alefts{i} ne ' ' then do ;
                        maxa = i ;
                        call symput('_atxt' || compress(i),trim(alefts{i})) ;
                        end ;
                    end;
                    
                    call symput('_maxline',put(maxh,2.)) ;
                    call symput('_maxfoot',put(maxf,2.)) ;
                    call symput('_maxtail',put(maxa,2.)) ;
                    
                end ;

            run ;


        /* set up flag to switch off rtfinit initializations of _nhead and _nfoot
        */
        %global
                _nhead
                _nfoot
                _tgenflg
                _ntail
                ;
        %let _nhead=0;
        %let _nfoot=0;
        %let _ntail =0;
        %let _tgenflg=1;
        
        

        %*** create the titles ;
        %do i=1 %to &_maxline ;

                %rtftitle(text=%bquote(&&_tctxt&i), ltext=%bquote(&&_tltxt&i), rtext=%bquote(&&_trtxt&i), 
                        lmarg=&tlmarg, rmarg=&trmarg, fontnum=&tfont, 
                        fontsize=&tfsize, just=c, bold=&tbold, under=N, italics=N) ;
                                
        %end ;
        
        %rtftitle(text= , ltext= , rtext= , lmarg=15, rmarg=15, fontnum=&tfont, 
                fontsize=&tfsize, just=c, bold=N, under=N, italics=N) ;

	%let _tottail = &_maxtail ;
        
        %do i=1 %to &_maxtail ;
        
                %rtftail(text=%bquote(&&_atxt&i), fontnum=&ffont, 
                        fontsize=&ffsize, just=l, bold=N, under=N, italics=N) ;
                        
        %end ;
               
	%let _totfoot = &_maxfoot ;
        
        %do i=1 %to &_maxfoot ;
        
                %rtffoot(text=%bquote(&&_fctxt&i), ltext=%bquote(&&_fltxt&i), rtext=%bquote(&&_frtxt&i), 
                        lmarg=&flmarg, rmarg=&frmarg, fontnum=&tfont, 
                        fontsize=&tfsize, just=c, bold=N, under=N, italics=N) ;

        %end ;
	

 
            %global printfil ;

            %if &filedest> %then %do;
                %let printfil = &filedest.\&progname..&ref ;
            %end;
            %else %do;
                %let printfil = &progname..&ref ;
            %end;


%mend rtfttlgn ;

